@@ -6,36 +6,40 @@ from collections.abc import Awaitable, Callable, Iterator
 from contextlib import contextmanager
 from functools import partial
 import importlib.resources
-from itertools import starmap
 import os
 from types import SimpleNamespace
 import typing
-from typing import Any, TypeVar, overload
+from typing import Any, ClassVar, TypeVar
 
 import aiohttp
 import aiohttp.web
-from aiohttp_rpc import JsonRpcMethod, WsJsonRpcServer, middlewares as rpc_middlewares
-from aiohttp_rpc.errors import InvalidParams as InvalidParamsError, ServerError
+from aiohttp_rpc import JsonRpcMethod
+from aiohttp_rpc import middlewares as rpc_middlewares
+from aiohttp_rpc.errors import InvalidParams as InvalidParamsError
+from aiohttp_rpc.errors import ServerError
+from aiohttp_rpc.server import WsJsonRpcServer
 import click
 from loguru import logger
 from pydantic import BaseModel, ValidationError
+import sqlalchemy as sa
 from typing_extensions import Concatenate, Literal, ParamSpec, TypeAlias, TypedDict
 from yarl import URL
 
-from instawow import __version__, matchers, results as R
+from instawow import __version__, db, matchers, models
+from instawow import results as R
+from instawow.common import Strategy
 from instawow.config import Config
-from instawow.manager import Manager, TraceRequestCtx, init_web_client, prepare_database
-from instawow.models import Pkg, is_pkg
-from instawow.resolvers import CatalogueEntry, Defn, PkgModel, Strategy
-from instawow.utils import is_outdated, run_in_thread as t, uniq
+from instawow.manager import Manager, TraceRequestCtx, init_web_client
+from instawow.resolvers import CatalogueEntry, Defn
+from instawow.utils import is_outdated
+from instawow.utils import run_in_thread as t
+from instawow.utils import uniq
 
-from . import InstawowApp, frontend, templates
+from . import frontend, templates
 
 _T = TypeVar('_T')
 _P = ParamSpec('_P')
-_ManagerWorkQueueItem: TypeAlias = (
-    'tuple[asyncio.Future[Any], str, Callable[..., Awaitable[Any]] | None]'
-)
+_ManagerWorkQueueItem: TypeAlias = 'tuple[asyncio.Future[Any], str, Callable[..., Awaitable[Any]]]'
 
 
 LOCALHOST = '127.0.0.1'
@@ -48,7 +52,8 @@ class _ConfigError(ServerError):
 
 @contextmanager
 def _reraise_validation_error(
-    error_class: type[ServerError] = ServerError, values: dict[Any, Any] | None = None
+    error_class: type[ServerError | InvalidParamsError] = ServerError,
+    values: dict[Any, Any] | None = None,
 ) -> Iterator[None]:
     try:
         yield
@@ -59,6 +64,8 @@ def _reraise_validation_error(
 
 
 class BaseParams(BaseModel):
+    method: ClassVar[str]
+
     async def respond(self, managers: _ManagerWorkQueue) -> Any:
         raise NotImplementedError
 
@@ -72,6 +79,8 @@ class _DefnParamMixin(BaseModel):
 
 
 class WriteConfigParams(BaseParams):
+    method = 'config/write'
+
     values: typing.Dict[str, Any]
     infer_game_flavour: bool
 
@@ -90,6 +99,8 @@ class WriteConfigParams(BaseParams):
 
 
 class ReadConfigParams(_ProfileParamMixin, BaseParams):
+    method = 'config/read'
+
     @t
     def respond(self, managers: _ManagerWorkQueue) -> Config:
         with _reraise_validation_error(_ConfigError):
@@ -97,6 +108,8 @@ class ReadConfigParams(_ProfileParamMixin, BaseParams):
 
 
 class DeleteConfigParams(_ProfileParamMixin, BaseParams):
+    method = 'config/delete'
+
     async def respond(self, managers: _ManagerWorkQueue) -> None:
         async def delete_profile(manager: Manager):
             await t(manager.config.delete)()
@@ -106,6 +119,8 @@ class DeleteConfigParams(_ProfileParamMixin, BaseParams):
 
 
 class ListProfilesParams(BaseParams):
+    method = 'config/list'
+
     @t
     def respond(self, managers: _ManagerWorkQueue) -> list[str]:
         return Config.list_profiles()
@@ -120,32 +135,38 @@ class Source(TypedDict):
 
 
 class ListSourcesParams(_ProfileParamMixin, BaseParams):
+    method = 'sources/list'
+
     async def respond(self, managers: _ManagerWorkQueue) -> list[Source]:
-        manager = await managers.run(self.profile)
+        manager = await managers.run(self.profile, _get_manager)
         return [
-            Source(
-                source=r.source,
-                name=r.name,
-                supported_strategies=sorted(r.strategies, key=list(Strategy).index),
-                supports_rollback=r.supports_rollback,
-                changelog_format=r.changelog_format.value,
-            )
+            {
+                'source': r.source,
+                'name': r.name,
+                'supported_strategies': sorted(r.strategies, key=list(Strategy).index),
+                'supports_rollback': Strategy.version in r.strategies,
+                'changelog_format': r.changelog_format.value,
+            }
             for r in manager.resolvers.values()
         ]
 
 
 class ListInstalledParams(_ProfileParamMixin, BaseParams):
-    async def respond(self, managers: _ManagerWorkQueue) -> list[PkgModel]:
-        from sqlalchemy import func
+    method = 'list'
 
-        manager = await managers.run(self.profile)
-        installed_pkgs = manager.database.query(Pkg).order_by(func.lower(Pkg.name)).all()
-        return [PkgModel.from_orm(p) for p in installed_pkgs]
+    async def respond(self, managers: _ManagerWorkQueue) -> list[models.Pkg]:
+        manager = await managers.run(self.profile, _get_manager)
+        installed_pkgs = (
+            manager.database.execute(sa.select(db.pkg).order_by(sa.func.lower(db.pkg.c.name)))
+            .mappings()
+            .all()
+        )
+        return [models.Pkg.from_row_mapping(manager.database, p) for p in installed_pkgs]
 
 
 class SuccessResult(TypedDict):
     status: Literal['success']
-    addon: PkgModel
+    addon: models.Pkg
 
 
 class ErrorResult(TypedDict):
@@ -154,6 +175,8 @@ class ErrorResult(TypedDict):
 
 
 class SearchParams(_ProfileParamMixin, BaseParams):
+    method = 'search'
+
     search_terms: str
     limit: int
     sources: typing.Optional[typing.Set[str]] = None
@@ -171,32 +194,32 @@ class SearchParams(_ProfileParamMixin, BaseParams):
 
 
 class ResolveParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
-    async def respond(self, managers: _ManagerWorkQueue) -> list[SuccessResult | ErrorResult]:
-        def extract_source(manager: Manager, defns: list[Defn]):
-            for defn in defns:
-                if defn.source == '*':
-                    pair = manager.pair_uri(defn.alias)
-                    if pair:
-                        source, alias = pair
-                        defn = defn.with_(source=source, alias=alias)
-                yield defn
+    method = 'resolve'
 
-        results = await managers.run(
-            self.profile,
-            partial(
-                Manager.resolve,
-                defns=list(extract_source(await managers.run(self.profile), self.defns)),
-            ),
-        )
+    async def _resolve(self, manager: Manager) -> dict[Defn, Any]:
+        def extract_source(defn: Defn):
+            if defn.source == '*':
+                pair = manager.pair_uri(defn.alias)
+                if pair:
+                    source, alias = pair
+                    defn = defn.with_(source=source, alias=alias)
+            return defn
+
+        return await manager.resolve(list(map(extract_source, self.defns)))
+
+    async def respond(self, managers: _ManagerWorkQueue) -> list[SuccessResult | ErrorResult]:
+        results = await managers.run(self.profile, self._resolve)
         return [
-            SuccessResult(status='success', addon=PkgModel.from_orm(r))
-            if is_pkg(r)
-            else ErrorResult(status=r.status, message=r.message)
+            {'status': 'success', 'addon': r}
+            if models.is_pkg(r)
+            else {'status': r.status, 'message': r.message}
             for r in results.values()
         ]
 
 
 class InstallParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
+    method = 'install'
+
     replace: bool
 
     async def respond(self, managers: _ManagerWorkQueue) -> list[SuccessResult | ErrorResult]:
@@ -204,27 +227,31 @@ class InstallParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
             self.profile, partial(Manager.install, defns=self.defns, replace=self.replace)
         )
         return [
-            SuccessResult(status=r.status, addon=PkgModel.from_orm(r.pkg))
+            {'status': r.status, 'addon': r.pkg}
             if isinstance(r, R.PkgInstalled)
-            else ErrorResult(status=r.status, message=r.message)
+            else {'status': r.status, 'message': r.message}
             for r in results.values()
         ]
 
 
 class UpdateParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
+    method = 'update'
+
     async def respond(self, managers: _ManagerWorkQueue) -> list[SuccessResult | ErrorResult]:
         results = await managers.run(
-            self.profile, partial(Manager.update, defns=self.defns, retain_strategy=True)
+            self.profile, partial(Manager.update, defns=self.defns, retain_defn_strategy=True)
         )
         return [
-            SuccessResult(status=r.status, addon=PkgModel.from_orm(r.new_pkg))
+            {'status': r.status, 'addon': r.new_pkg}
             if isinstance(r, R.PkgUpdated)
-            else ErrorResult(status=r.status, message=r.message)
+            else {'status': r.status, 'message': r.message}
             for r in results.values()
         ]
 
 
 class RemoveParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
+    method = 'remove'
+
     keep_folders: bool
 
     async def respond(self, managers: _ManagerWorkQueue) -> list[SuccessResult | ErrorResult]:
@@ -232,25 +259,29 @@ class RemoveParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
             self.profile, partial(Manager.remove, defns=self.defns, keep_folders=self.keep_folders)
         )
         return [
-            SuccessResult(status=r.status, addon=PkgModel.from_orm(r.old_pkg))
+            {'status': r.status, 'addon': r.old_pkg}
             if isinstance(r, R.PkgRemoved)
-            else ErrorResult(status=r.status, message=r.message)
+            else {'status': r.status, 'message': r.message}
             for r in results.values()
         ]
 
 
 class PinParams(_ProfileParamMixin, _DefnParamMixin, BaseParams):
+    method = 'pin'
+
     async def respond(self, managers: _ManagerWorkQueue) -> list[SuccessResult | ErrorResult]:
         results = await managers.run(self.profile, partial(Manager.pin, defns=self.defns))
         return [
-            SuccessResult(status=r.status, addon=PkgModel.from_orm(r.pkg))
+            {'status': r.status, 'addon': r.pkg}
             if isinstance(r, R.PkgInstalled)
-            else ErrorResult(status=r.status, message=r.message)
+            else {'status': r.status, 'message': r.message}
             for r in results.values()
         ]
 
 
 class GetChangelogParams(_ProfileParamMixin, BaseParams):
+    method = 'get_changelog'
+
     changelog_url: str
 
     async def respond(self, managers: _ManagerWorkQueue) -> str:
@@ -259,22 +290,24 @@ class GetChangelogParams(_ProfileParamMixin, BaseParams):
         )
 
 
-class ReconcileResult_AddonFolder(TypedDict):
-    name: str
-    version: str
-
-
-class ReconcileResult_AddonMatch(TypedDict):
-    folders: list[ReconcileResult_AddonFolder]
-    matches: list[PkgModel]
-
-
 class ReconcileResult(TypedDict):
     reconciled: list[ReconcileResult_AddonMatch]
     unreconciled: list[ReconcileResult_AddonMatch]
 
 
+class ReconcileResult_AddonMatch(TypedDict):
+    folders: list[ReconcileResult_AddonFolder]
+    matches: list[models.Pkg]
+
+
+class ReconcileResult_AddonFolder(TypedDict):
+    name: str
+    version: str
+
+
 class ReconcileParams(_ProfileParamMixin, BaseParams):
+    method = 'reconcile'
+
     matcher: Literal['toc_source_ids', 'folder_name_subsets', 'addon_names_with_folder_names']
 
     async def respond(self, managers: _ManagerWorkQueue) -> ReconcileResult:
@@ -286,26 +319,22 @@ class ReconcileParams(_ProfileParamMixin, BaseParams):
         resolve_results = await managers.run(
             self.profile, partial(Manager.resolve, defns=uniq_defns)
         )
-        return ReconcileResult(
-            reconciled=[
-                ReconcileResult_AddonMatch(
-                    folders=[
-                        ReconcileResult_AddonFolder(name=f.name, version=f.version) for f in a
-                    ],
-                    matches=[
-                        PkgModel.from_orm(r) for i in d for r in (resolve_results[i],) if is_pkg(r)
-                    ],
-                )
+        return {
+            'reconciled': [
+                {
+                    'folders': [{'name': f.name, 'version': f.version} for f in a],
+                    'matches': [r for i in d for r in (resolve_results[i],) if models.is_pkg(r)],
+                }
                 for a, d in match_groups
             ],
-            unreconciled=[
-                ReconcileResult_AddonMatch(
-                    folders=[ReconcileResult_AddonFolder(name=f.name, version=f.version)],
-                    matches=[],
-                )
+            'unreconciled': [
+                {
+                    'folders': [{'name': f.name, 'version': f.version}],
+                    'matches': [],
+                }
                 for f in sorted(leftovers - frozenset(i for a, _ in match_groups for i in a))
             ],
-        )
+        }
 
 
 class DownloadProgressReport(TypedDict):
@@ -314,9 +343,11 @@ class DownloadProgressReport(TypedDict):
 
 
 class GetDownloadProgressParams(_ProfileParamMixin, BaseParams):
+    method = 'get_download_progress'
+
     async def respond(self, managers: _ManagerWorkQueue) -> list[DownloadProgressReport]:
         return [
-            DownloadProgressReport(defn=Defn.from_pkg(p), progress=r)
+            {'defn': Defn.from_pkg(p), 'progress': r}
             for p, r in await managers.get_download_progress(self.profile)
         ]
 
@@ -327,14 +358,19 @@ class GetVersionResult(TypedDict):
 
 
 class GetVersionParams(BaseParams):
+    method = 'meta/get_version'
+
     async def respond(self, managers: _ManagerWorkQueue) -> GetVersionResult:
         outdated, new_version = await is_outdated()
-        return GetVersionResult(
-            installed_version=__version__, new_version=new_version if outdated else None
-        )
+        return {
+            'installed_version': __version__,
+            'new_version': new_version if outdated else None,
+        }
 
 
 class OpenUrlParams(BaseParams):
+    method = 'assist/open_url'
+
     url: str
 
     async def respond(self, managers: _ManagerWorkQueue) -> None:
@@ -342,6 +378,8 @@ class OpenUrlParams(BaseParams):
 
 
 class RevealFolderParams(BaseParams):
+    method = 'assist/reveal_folder'
+
     path_parts: typing.List[str]
 
     async def respond(self, managers: _ManagerWorkQueue) -> None:
@@ -353,22 +391,20 @@ class SelectFolderResult(TypedDict):
 
 
 class SelectFolderParams(BaseParams):
+    method = 'assist/select_folder'
+
     initial_folder: typing.Optional[str]
 
     async def respond(self, managers: _ManagerWorkQueue) -> SelectFolderResult:
-        async def select_folder():
-            try:
-                (selection,) = InstawowApp.app.iw_window.select_folder_dialog(
-                    'Select folder', self.initial_folder
-                )
-            except ValueError:
-                selection = None
-            return selection
+        from .app import InstawowApp
 
-        selection = asyncio.run_coroutine_threadsafe(
-            select_folder(), InstawowApp.app.loop
-        ).result()
-        return SelectFolderResult(selection=selection)
+        try:
+            (selection,) = InstawowApp.app.iw_window.select_folder_dialog(
+                'Select folder', self.initial_folder
+            )
+        except ValueError:
+            selection = None
+        return {'selection': selection}
 
 
 class ConfirmDialogueResult(TypedDict):
@@ -376,19 +412,20 @@ class ConfirmDialogueResult(TypedDict):
 
 
 class ConfirmDialogueParams(BaseParams):
+    method = 'assist/confirm'
+
     title: str
     message: str
 
     async def respond(self, managers: _ManagerWorkQueue) -> ConfirmDialogueResult:
-        async def confirm():
-            return InstawowApp.app.iw_window.confirm_dialog(self.title, self.message)
+        from .app import InstawowApp
 
-        ok = asyncio.run_coroutine_threadsafe(confirm(), InstawowApp.app.loop).result()
-        return ConfirmDialogueResult(ok=ok)
+        ok = InstawowApp.app.iw_window.confirm_dialog(self.title, self.message)
+        return {'ok': ok}
 
 
 def _init_json_rpc_web_client(
-    progress_reporters: set[tuple[Manager, Pkg, Callable[[], float]]],
+    progress_reporters: set[tuple[Manager, models.Pkg, Callable[[], float]]],
 ) -> aiohttp.ClientSession:
     async def do_on_request_end(
         client_session: aiohttp.ClientSession,
@@ -418,92 +455,75 @@ def _init_json_rpc_web_client(
     return init_web_client(trace_configs=[trace_config])
 
 
+async def _get_manager(manager: Manager):
+    return manager
+
+
 class _ManagerWorkQueue:
     def __init__(self) -> None:
+        self._loop = asyncio.get_running_loop()
         self._managers: dict[str, Manager] = {}
         self._queue: asyncio.Queue[_ManagerWorkQueueItem] = asyncio.Queue()
-        self._progress_reporters: set[tuple[Manager, Pkg, Callable[[], float]]] = set()
+        self._progress_reporters: set[tuple[Manager, models.Pkg, Callable[[], float]]] = set()
         self._web_client = _init_json_rpc_web_client(self._progress_reporters)
         self._locks: defaultdict[str, asyncio.Lock] = defaultdict(asyncio.Lock)
 
     async def cleanup(self) -> None:
+        for manager in self._managers.values():
+            manager.database.close()
         await self._web_client.close()
 
+    def unload(self, profile: str) -> None:
+        manager = self._managers.pop(profile, None)
+        if manager:
+            manager.database.close()
+
     async def listen(self) -> None:
+        async def schedule(
+            future: asyncio.Future[Any], profile: str, coro_fn: Callable[..., Awaitable[Any]]
+        ):
+            try:
+                async with self._locks[f"load profile '{profile}'"]:
+                    try:
+                        manager = self._managers[profile]
+                    except KeyError:
+                        with _reraise_validation_error(_ConfigError):
+                            config = await t(Config.read)(profile)
+
+                        manager = self._managers[profile] = Manager.from_config(config)
+
+                result = await coro_fn(manager)
+            except BaseException as exc:
+                future.set_exception(exc)
+            else:
+                future.set_result(result)
+
         Manager.contextualise(web_client=self._web_client, locks=self._locks)
 
         while True:
             item = await self._queue.get()
-
-            async def schedule(item: _ManagerWorkQueueItem):
-                future, profile, coro_fn = item
-                try:
-                    async with self._locks[f"load profile '{profile}'"]:
-                        try:
-                            manager = self._managers[profile]
-                        except KeyError:
-                            with _reraise_validation_error(_ConfigError):
-                                config = await t(Config.read)(profile)
-
-                            manager = self._managers[profile] = Manager(
-                                config=config,
-                                database=await t(prepare_database)(config),
-                            )
-
-                    if coro_fn is None:
-                        result = manager
-                    else:
-                        result = await coro_fn(manager)
-                except BaseException as exc:
-                    future.set_exception(exc)
-                else:
-                    future.set_result(result)
-
-            asyncio.create_task(schedule(item))
+            asyncio.create_task(schedule(*item))
             self._queue.task_done()
 
-    @overload
     async def run(
-        self,
-        profile: str,
-        coro_fn: None = None,
-    ) -> Manager:
-        ...
-
-    @overload
-    async def run(
-        self,
-        profile: str,
-        coro_fn: Callable[Concatenate[Manager, _P], Awaitable[_T]],
+        self, profile: str, coro_fn: Callable[Concatenate[Manager, _P], Awaitable[_T]]
     ) -> _T:
-        ...
-
-    async def run(
-        self,
-        profile: str,
-        coro_fn: Callable[Concatenate[Manager, _P], Awaitable[_T]] | None = None,
-    ) -> _T | Manager:
-        future: asyncio.Future[Any] = asyncio.Future()
+        future = self._loop.create_future()
         self._queue.put_nowait((future, profile, coro_fn))
         return await asyncio.wait_for(future, None)
 
-    async def get_download_progress(self, profile: str) -> Iterator[tuple[Pkg, float]]:
-        manager = await self.run(profile)
+    async def get_download_progress(self, profile: str) -> Iterator[tuple[models.Pkg, float]]:
+        manager = await self.run(profile, _get_manager)
         return ((p, r()) for m, p, r in self._progress_reporters if m is manager)
 
-    def unload(self, profile: str) -> None:
-        self._managers.pop(profile, None)
 
-
-def _prepare_response(
-    param_class: type[BaseParams], method: str, managers: _ManagerWorkQueue
-) -> JsonRpcMethod:
+def _prepare_response(param_class: type[BaseParams], managers: _ManagerWorkQueue) -> JsonRpcMethod:
     async def respond(**kwargs: Any) -> BaseModel:
         with _reraise_validation_error(InvalidParamsError, kwargs):
             params = param_class.parse_obj(kwargs)
         return await params.respond(managers)
 
-    return JsonRpcMethod('', respond, custom_name=method)
+    return JsonRpcMethod(respond, name=param_class.method)
 
 
 def _serialise_response(value: dict[str, Any]) -> str:
@@ -512,15 +532,13 @@ def _serialise_response(value: dict[str, Any]) -> str:
 
 async def create_app() -> aiohttp.web.Application:
     managers = _ManagerWorkQueue()
+    prepare_response = partial(_prepare_response, managers=managers)
 
-    def start_managers():
+    async def start_managers(app: aiohttp.web.Application):
         managers_listen = asyncio.create_task(managers.listen())
-
-        async def on_shutdown(app: aiohttp.web.Application):
-            managers_listen.cancel()
-            await managers.cleanup()
-
-        return on_shutdown
+        yield
+        managers_listen.cancel()
+        await managers.cleanup()
 
     async def get_index(request: aiohttp.web.Request):
         return aiohttp.web.Response(
@@ -542,6 +560,35 @@ async def create_app() -> aiohttp.web.Application:
             text=importlib.resources.read_text(frontend, filename),
         )
 
+    rpc_server = WsJsonRpcServer(
+        json_serialize=_serialise_response,
+        middlewares=rpc_middlewares.DEFAULT_MIDDLEWARES,
+    )
+    rpc_server.add_methods(
+        [
+            prepare_response(WriteConfigParams),
+            prepare_response(ReadConfigParams),
+            prepare_response(DeleteConfigParams),
+            prepare_response(ListProfilesParams),
+            prepare_response(ListSourcesParams),
+            prepare_response(ListInstalledParams),
+            prepare_response(SearchParams),
+            prepare_response(ResolveParams),
+            prepare_response(InstallParams),
+            prepare_response(UpdateParams),
+            prepare_response(RemoveParams),
+            prepare_response(PinParams),
+            prepare_response(GetChangelogParams),
+            prepare_response(ReconcileParams),
+            prepare_response(GetDownloadProgressParams),
+            prepare_response(GetVersionParams),
+            prepare_response(OpenUrlParams),
+            prepare_response(RevealFolderParams),
+            prepare_response(SelectFolderParams),
+            prepare_response(ConfirmDialogueParams),
+        ]
+    )
+
     app = aiohttp.web.Application()
     app.add_routes(
         [
@@ -549,43 +596,10 @@ async def create_app() -> aiohttp.web.Application:
             aiohttp.web.get(
                 r'/svelte-bundle{extension:(?:\.css|\.js(?:\.map)?)}', get_static_file
             ),
+            aiohttp.web.get('/api', rpc_server.handle_http_request),
         ]
     )
-
-    rpc_server = WsJsonRpcServer(
-        json_serialize=_serialise_response,
-        middlewares=rpc_middlewares.DEFAULT_MIDDLEWARES,
-    )
-    rpc_server.add_methods(
-        starmap(
-            _prepare_response,
-            [
-                (WriteConfigParams, 'config/write', managers),
-                (ReadConfigParams, 'config/read', managers),
-                (DeleteConfigParams, 'config/delete', managers),
-                (ListProfilesParams, 'config/list', managers),
-                (ListSourcesParams, 'sources/list', managers),
-                (ListInstalledParams, 'list', managers),
-                (SearchParams, 'search', managers),
-                (ResolveParams, 'resolve', managers),
-                (InstallParams, 'install', managers),
-                (UpdateParams, 'update', managers),
-                (RemoveParams, 'remove', managers),
-                (PinParams, 'pin', managers),
-                (GetChangelogParams, 'get_changelog', managers),
-                (ReconcileParams, 'reconcile', managers),
-                (GetDownloadProgressParams, 'get_download_progress', managers),
-                (GetVersionParams, 'meta/get_version', managers),
-                (OpenUrlParams, 'assist/open_url', managers),
-                (RevealFolderParams, 'assist/reveal_folder', managers),
-                (SelectFolderParams, 'assist/select_folder', managers),
-                (ConfirmDialogueParams, 'assist/confirm', managers),
-            ],
-        )
-    )
-    app.add_routes([aiohttp.web.get('/api', rpc_server.handle_http_request)])
-
-    app.on_shutdown.append(start_managers())
+    app.cleanup_ctx.append(start_managers)
     app.freeze()
     return app
 
