@@ -1,21 +1,31 @@
 from __future__ import annotations
 
-from collections.abc import Callable, Generator, Iterable, Iterator, Sequence, Set
+import asyncio
+from collections.abc import Awaitable, Callable, Generator, Iterable, Iterator, Sequence, Set
+from contextlib import contextmanager
 from datetime import datetime
 from enum import Enum
-from functools import partial
+from functools import partial, wraps
+import importlib.util
 from itertools import chain
 from pathlib import Path
 import textwrap
-from typing import overload
+from typing import Any, NoReturn, TypeVar, overload
 
 import click
 
-from . import __version__, manager as managers, models, results as R
+from . import __version__, _deferred_types, db
+from . import manager as _manager
+from . import models
+from . import results as R
+from .common import Strategy
 from .config import Config, Flavour, setup_logging
 from .plugins import load_plugins
-from .resolvers import Defn, MultiPkgModel, Strategy
-from .utils import cached_property, is_outdated, tabulate, uniq
+from .resolvers import ChangelogFormat, Defn
+from .utils import cached_property, gather, is_outdated, make_progress_bar, tabulate, uniq
+
+_T = TypeVar('_T')
+_F = TypeVar('_F', bound='Callable[..., object]')
 
 
 class Report:
@@ -49,7 +59,7 @@ class Report:
 
     def __str__(self) -> str:
         return '\n'.join(
-            f'{self._result_type_to_symbol(r)} {click.style(a.to_uri(), bold=True)}\n'
+            f'{self._result_type_to_symbol(r)} {click.style(a.to_urn(), bold=True)}\n'
             f'{textwrap.fill(r.message, initial_indent=" " * 2, subsequent_indent=" " * 4)}'
             for a, r in self.results
             if self.filter_fn(r)
@@ -57,8 +67,8 @@ class Report:
 
     def generate(self) -> None:
         manager_wrapper: ManagerWrapper | None = click.get_current_context().obj
-        if manager_wrapper and manager_wrapper.m.config.auto_update_check:
-            outdated, new_version = manager_wrapper.m.run(is_outdated())
+        if manager_wrapper and manager_wrapper.manager.config.auto_update_check:
+            outdated, new_version = run_with_progress(is_outdated())
             if outdated:
                 click.echo(f'{self.WARNING_SYMBOL} instawow-{new_version} is available')
 
@@ -66,41 +76,92 @@ class Report:
         if report:
             click.echo(report)
 
-    def generate_and_exit(self) -> None:
+    def generate_and_exit(self) -> NoReturn:
         self.generate()
-        ctx = click.get_current_context()
+        ctx: click.Context = click.get_current_context()
         ctx.exit(self.exit_code)
 
 
-def _set_mac_multiprocessing_start_method() -> None:
-    import sys
+def _init_cli_web_client(
+    progress_bar: _deferred_types.prompt_toolkit.shortcuts.ProgressBar,
+    tickers: set[asyncio.Task[None]],
+) -> _deferred_types.aiohttp.ClientSession:
+    from aiohttp import TraceConfig, hdrs
+    from prompt_toolkit.shortcuts import ProgressBarCounter
 
-    # Reference: https://github.com/indygreg/PyOxidizer/issues/111#issuecomment-808834727
+    TICK_INTERVAL = 0.1
 
-    if getattr(sys, 'frozen', False) and sys.platform == 'darwin':
-        import multiprocessing
+    async def do_on_request_end(
+        client_session: _deferred_types.aiohttp.ClientSession,
+        trace_config_ctx: Any,
+        params: _deferred_types.aiohttp.TraceRequestEndParams,
+    ):
+        trace_request_ctx: _manager.TraceRequestCtx = trace_config_ctx.trace_request_ctx
+        if trace_request_ctx:
+            response = params.response
+            label = (
+                'Downloading '
+                + Defn(trace_request_ctx['pkg'].source, trace_request_ctx['pkg'].slug).to_urn()
+                if trace_request_ctx['report_progress'] == 'pkg_download'
+                else trace_request_ctx['label']
+            )
+            # The encoded size is not exposed in the aiohttp streaming API
+            # so we cannot display progress.  When the total is ``None``
+            # the progress bar is animated as being "indeterminate"
+            total = None if hdrs.CONTENT_ENCODING in response.headers else response.content_length
 
-        multiprocessing.set_start_method('fork')
+            async def ticker():
+                counter: ProgressBarCounter[None] = ProgressBarCounter(
+                    progress_bar=progress_bar, label=label, total=total
+                )
+                progress_bar.counters.append(counter)
+                try:
+                    while not response.content.is_eof():
+                        counter.items_completed = response.content.total_bytes
+                        progress_bar.invalidate()
+                        await asyncio.sleep(TICK_INTERVAL)
+                finally:
+                    progress_bar.counters.remove(counter)
+
+            tickers.add(asyncio.create_task(ticker()))
+
+    trace_config = TraceConfig()
+    trace_config.on_request_end.append(do_on_request_end)
+    trace_config.freeze()
+    return _manager.init_web_client(trace_configs=[trace_config])
+
+
+@contextmanager
+def _cancel_tickers() -> Iterator[set[asyncio.Task[None]]]:
+    tickers: set[asyncio.Task[None]] = set()
+    try:
+        yield tickers
+    finally:
+        for ticker in tickers:
+            ticker.cancel()
+
+
+def run_with_progress(awaitable: Awaitable[_T]) -> _T:
+    async def run():
+        with _cancel_tickers() as tickers, make_progress_bar() as progress_bar:
+            async with _init_cli_web_client(progress_bar, tickers) as web_client:
+                _manager.Manager.contextualise(web_client=web_client)
+                return await awaitable
+
+    return asyncio.run(run())
 
 
 def _override_asyncio_loop_policy() -> None:
     # The proactor event loop which became the default loop on Windows
     # in Python 3.8 is causing issues with aiohttp.
     # See https://github.com/aio-libs/aiohttp/issues/4324
-    import asyncio
-
     policy = getattr(asyncio, 'WindowsSelectorEventLoopPolicy', None)
     if policy:
         asyncio.set_event_loop_policy(policy())
 
 
-def _set_asyncio_debug(debug: bool) -> None:
-    # This is the least obtrusive way to enable debugging in asyncio.
-    # Reference: https://docs.python.org/3/library/asyncio-dev.html#debug-mode
-    if debug:
-        import os
-
-        os.environ['PYTHONASYNCIODEBUG'] = '1'
+def _apply_patches() -> None:
+    _override_asyncio_loop_policy()
 
 
 class ManagerWrapper:
@@ -108,28 +169,32 @@ class ManagerWrapper:
         self.ctx = ctx
 
     @cached_property
-    def m(self) -> managers.CliManager:
-        _set_mac_multiprocessing_start_method()
-        _override_asyncio_loop_policy()
-        _set_asyncio_debug(self.ctx.params['log_level'] == 'DEBUG')
-
+    def manager(self) -> _manager.Manager:
         try:
             config = Config.read(self.ctx.params['profile']).ensure_dirs()
         except FileNotFoundError:
             config = self.ctx.invoke(configure, promptless=False)
 
-        setup_logging(config, self.ctx.params['log_level'])
+        setup_logging(config, self.ctx.params['log_level'], self.ctx.params['log_to_stderr'])
 
-        manager = managers.CliManager.from_config(config)
+        manager = _manager.Manager.from_config(config)
+        self.ctx.call_on_close(manager.database.close)
         return manager
 
+    @staticmethod
+    def pass_manager(fn: _F) -> _F:
+        @wraps(fn)
+        def wrapper(*args: object, **kwargs: object):
+            return fn(click.get_current_context().obj.manager, *args, **kwargs)
+
+        return wrapper  # type: ignore
+
 
 def _with_manager(
     fn: Callable[..., object]
 ) -> Callable[[click.Context, click.Parameter, object], object]:
-    def wrapper(ctx: click.Context, __: click.Parameter, value: object) -> object:
-        assert ctx.obj
-        return fn(ctx.obj.m, value)
+    def wrapper(ctx: click.Context, __: click.Parameter, value: object):
+        return fn(ctx.obj.manager, value)
 
     return wrapper
 
@@ -177,6 +242,13 @@ def _set_log_level(_: click.Context, __: click.Parameter, value: bool) -> str:
     callback=_set_log_level,
     help='Log more things.',
 )
+@click.option(
+    '--log-to-stderr',
+    is_flag=True,
+    default=False,
+    help='Log to stderr for development.',
+    hidden=True,
+)
 @click.option(
     '--profile',
     '-p',
@@ -184,26 +256,35 @@ def _set_log_level(_: click.Context, __: click.Parameter, value: bool) -> str:
     help='Activate the specified profile.',
 )
 @click.pass_context
-def main(ctx: click.Context, log_level: str, profile: str) -> None:
+def main(ctx: click.Context, log_level: str, log_to_stderr: bool, profile: str) -> None:
     "Add-on manager for World of Warcraft."
+    _apply_patches()
     ctx.obj = ManagerWrapper(ctx)
 
 
 @overload
-def parse_into_defn(manager: managers.Manager, value: str, *, raise_invalid: bool = True) -> Defn:
+def parse_into_defn(manager: _manager.Manager, value: str, *, raise_invalid: bool = True) -> Defn:
     ...
 
 
 @overload
 def parse_into_defn(
-    manager: managers.Manager, value: list[str], *, raise_invalid: bool = True
+    manager: _manager.Manager, value: list[str], *, raise_invalid: bool = True
 ) -> list[Defn]:
     ...
 
 
+@overload
+def parse_into_defn(manager: _manager.Manager, value: None, *, raise_invalid: bool = True) -> None:
+    ...
+
+
 def parse_into_defn(
-    manager: managers.Manager, value: str | list[str], *, raise_invalid: bool = True
-) -> Defn | list[Defn]:
+    manager: _manager.Manager, value: str | list[str] | None, *, raise_invalid: bool = True
+) -> Defn | list[Defn] | None:
+    if value is None:
+        return None
+
     if not isinstance(value, str):
         defns = (parse_into_defn(manager, v, raise_invalid=raise_invalid) for v in value)
         return uniq(defns)
@@ -213,34 +294,33 @@ def parse_into_defn(
         if raise_invalid:
             raise click.BadParameter(value)
 
-        pair = '*', value
+        pair = ('*', value)
     return Defn(*pair)
 
 
 def parse_into_defn_with_strategy(
-    manager: managers.Manager, value: Sequence[tuple[Strategy, str]]
+    manager: _manager.Manager, value: Sequence[tuple[Strategy, str]]
 ) -> Iterator[Defn]:
     defns = parse_into_defn(manager, [d for _, d in value])
-    return map(Defn.with_strategy, defns, (s for s, _ in value))
+    return map(lambda d, s: d.with_(strategy=s), defns, (s for s, _ in value))
 
 
 def parse_into_defn_with_version(
-    manager: managers.Manager, value: Sequence[tuple[str, str]]
+    manager: _manager.Manager, value: Sequence[tuple[str, str]]
 ) -> Iterator[Defn]:
     defns = parse_into_defn(manager, [d for _, d in value])
     return map(Defn.with_version, defns, (v for v, _ in value))
 
 
-def combine_addons(
-    fn: Callable[[managers.Manager, object], Iterable[Defn]],
+def _combine_addons(
+    fn: Callable[[_manager.Manager, object], Iterable[Defn]],
     ctx: click.Context,
     __: click.Parameter,
     value: object,
 ) -> None:
     addons: list[Defn] = ctx.params.setdefault('addons', [])
     if value:
-        assert ctx.obj
-        addons.extend(fn(ctx.obj.m, value))
+        addons.extend(fn(ctx.obj.manager, value))
 
 
 _EXCLUDED_STRATEGIES = frozenset({Strategy.default, Strategy.version})
@@ -248,7 +328,7 @@ _EXCLUDED_STRATEGIES = frozenset({Strategy.default, Strategy.version})
 
 @main.command()
 @click.argument(
-    'addons', nargs=-1, callback=partial(combine_addons, parse_into_defn), expose_value=False
+    'addons', nargs=-1, callback=partial(_combine_addons, parse_into_defn), expose_value=False
 )
 @click.option(
     '--with-strategy',
@@ -256,7 +336,7 @@ _EXCLUDED_STRATEGIES = frozenset({Strategy.default, Strategy.version})
     multiple=True,
     type=(EnumParam(Strategy, _EXCLUDED_STRATEGIES), str),
     expose_value=False,
-    callback=partial(combine_addons, parse_into_defn_with_strategy),
+    callback=partial(_combine_addons, parse_into_defn_with_strategy),
     metavar='<STRATEGY ADDON>...',
     help='A strategy followed by an add-on definition.  '
     'The strategy is one of: '
@@ -267,28 +347,29 @@ _EXCLUDED_STRATEGIES = frozenset({Strategy.default, Strategy.version})
     multiple=True,
     type=(str, str),
     expose_value=False,
-    callback=partial(combine_addons, parse_into_defn_with_version),
+    callback=partial(_combine_addons, parse_into_defn_with_version),
     metavar='<VERSION ADDON>...',
     help='A version followed by an add-on definition.',
 )
 @click.option('--replace', is_flag=True, default=False, help='Replace unreconciled add-ons.')
-@click.pass_obj
-def install(obj: ManagerWrapper, addons: Sequence[Defn], replace: bool) -> None:
+@ManagerWrapper.pass_manager
+def install(manager: _manager.Manager, addons: Sequence[Defn], replace: bool) -> None:
     "Install add-ons."
     if not addons:
         raise click.UsageError(
             'You must provide at least one of "ADDONS", "--with-strategy" or "--version"'
         )
 
-    results = obj.m.run(obj.m.install(addons, replace))
+    results = run_with_progress(manager.install(addons, replace))
     Report(results.items()).generate_and_exit()
 
 
 @main.command()
 @click.argument('addons', nargs=-1, callback=_with_manager(parse_into_defn))
-@click.pass_obj
-def update(obj: ManagerWrapper, addons: Sequence[Defn]) -> None:
+@ManagerWrapper.pass_manager
+def update(manager: _manager.Manager, addons: Sequence[Defn]) -> None:
     "Update installed add-ons."
+    import sqlalchemy as sa
 
     def filter_results(result: R.ManagerResult):
         # Hide packages from output if they are up to date
@@ -299,11 +380,21 @@ def update(obj: ManagerWrapper, addons: Sequence[Defn]) -> None:
         else:
             return result.is_pinned
 
-    results = obj.m.run(
-        obj.m.update(
-            addons or list(map(Defn.from_pkg, obj.m.database.query(models.Pkg).all())), False
+    update_defns = addons or [
+        Defn(**v)
+        for v in manager.database.execute(
+            sa.select(
+                db.pkg.c.source,
+                db.pkg.c.id,
+                db.pkg.c.slug.label('alias'),
+                db.pkg.c.version,
+                db.pkg_options.c.strategy,
+            ).join(db.pkg_options)
         )
-    )
+        .mappings()
+        .all()
+    ]
+    results = run_with_progress(manager.update(update_defns, False))
     Report(results.items(), filter_results).generate_and_exit()
 
 
@@ -315,10 +406,10 @@ def update(obj: ManagerWrapper, addons: Sequence[Defn]) -> None:
     default=False,
     help="Do not delete the add-on folders.",
 )
-@click.pass_obj
-def remove(obj: ManagerWrapper, addons: Sequence[Defn], keep_folders: bool) -> None:
+@ManagerWrapper.pass_manager
+def remove(manager: _manager.Manager, addons: Sequence[Defn], keep_folders: bool) -> None:
     "Remove add-ons."
-    results = obj.m.run(obj.m.remove(addons, keep_folders))
+    results = run_with_progress(manager.remove(addons, keep_folders))
     Report(results.items()).generate_and_exit()
 
 
@@ -334,28 +425,24 @@ def remove(obj: ManagerWrapper, addons: Sequence[Defn], keep_folders: bool) -> N
     default=False,
     help='Undo rollback by reinstalling an add-on using the default strategy.',
 )
-@click.pass_obj
-def rollback(obj: ManagerWrapper, addon: Defn, version: str | None, undo: bool) -> None:
+@ManagerWrapper.pass_manager
+def rollback(manager: _manager.Manager, addon: Defn, version: str | None, undo: bool) -> None:
     "Roll an add-on back to an older version."
     from .prompts import Choice, select
 
-    manager = obj.m
-
     if version and undo:
         raise click.UsageError('Cannot use "--version" and "--undo" together')
 
     pkg = manager.get_pkg(addon)
     if not pkg:
-        Report([(addon, R.PkgNotInstalled())]).generate_and_exit()
-        return  # pragma: no cover
+        report: Report = Report([(addon, R.PkgNotInstalled())])
+        report.generate_and_exit()
 
-    if not manager.resolvers[pkg.source].supports_rollback:
-        Report(
-            [(addon, R.PkgFileUnavailable('source does not support rollback'))]
-        ).generate_and_exit()
+    if Strategy.version not in manager.resolvers[pkg.source].strategies:
+        Report([(addon, R.PkgStrategyUnsupported(Strategy.version))]).generate_and_exit()
 
     if undo:
-        Report(manager.run(manager.update([addon], True)).items()).generate_and_exit()
+        Report(run_with_progress(manager.update([addon], True)).items()).generate_and_exit()
 
     reconstructed_defn = Defn.from_pkg(pkg)
     if version:
@@ -376,11 +463,13 @@ def rollback(obj: ManagerWrapper, addon: Defn, version: str | None, undo: bool)
             for v in versions
         ]
         selection = select(
-            f'Select version of {reconstructed_defn.to_uri()} for rollback', choices
+            f'Select version of {reconstructed_defn.to_urn()} for rollback', choices
         ).unsafe_ask()
 
     Report(
-        manager.run(manager.update([reconstructed_defn.with_version(selection)], True)).items()
+        run_with_progress(
+            manager.update([reconstructed_defn.with_version(selection)], True)
+        ).items()
     ).generate_and_exit()
 
 
@@ -391,8 +480,8 @@ def rollback(obj: ManagerWrapper, addon: Defn, version: str | None, undo: bool)
 @click.option(
     '--list-unreconciled', is_flag=True, default=False, help='List unreconciled add-ons and exit.'
 )
-@click.pass_obj
-def reconcile(obj: ManagerWrapper, auto: bool, list_unreconciled: bool) -> None:
+@ManagerWrapper.pass_manager
+def reconcile(manager: _manager.Manager, auto: bool, list_unreconciled: bool) -> None:
     "Reconcile pre-installed add-ons."
     from .matchers import (
         AddonFolder,
@@ -422,13 +511,11 @@ def reconcile(obj: ManagerWrapper, auto: bool, list_unreconciled: bool) -> None:
         '''
     )
 
-    manager = obj.m
-
     def prompt_one(addons: list[AddonFolder], pkgs: list[models.Pkg]) -> Defn | tuple[()]:
         def construct_choice(pkg: models.Pkg):
             defn = Defn.from_pkg(pkg)
             title = [
-                ('', f'{defn.to_uri()}=='),
+                ('', f'{defn.to_urn()}=='),
                 ('class:highlight-sub' if highlight_version else '', pkg.version),
             ]
             return PkgChoice(title, pkg=pkg, value=defn)
@@ -443,7 +530,7 @@ def reconcile(obj: ManagerWrapper, auto: bool, list_unreconciled: bool) -> None:
 
     def prompt(groups: Sequence[tuple[list[AddonFolder], list[Defn]]]) -> Iterable[Defn]:
         uniq_defns = uniq(d for _, b in groups for d in b)
-        results = manager.run(manager.resolve(uniq_defns))
+        results = run_with_progress(manager.resolve(uniq_defns))
         for addons, defns in groups:
             shortlist: list[models.Pkg] = list(filter(models.is_pkg, (results[d] for d in defns)))
             if shortlist:
@@ -457,7 +544,7 @@ def reconcile(obj: ManagerWrapper, auto: bool, list_unreconciled: bool) -> None:
             match_folder_name_subsets,
             match_addon_names_with_folder_names,
         ):
-            groups = manager.run(fn(manager, (yield [])))
+            groups = run_with_progress(fn(manager, (yield [])))
             yield list(prompt(groups))
 
     leftovers = get_unreconciled_folder_set(manager)
@@ -476,7 +563,7 @@ def reconcile(obj: ManagerWrapper, auto: bool, list_unreconciled: bool) -> None:
     for _ in matcher:  # Skip over consumer yields
         selections = matcher.send(leftovers)
         if selections and (auto or confirm('Install selected add-ons?').unsafe_ask()):
-            results = manager.run(manager.install(selections, replace=True))
+            results = run_with_progress(manager.install(selections, replace=True))
             Report(results.items()).generate()
 
         leftovers = get_unreconciled_folder_set(manager)
@@ -511,17 +598,16 @@ def search(ctx: click.Context, search_terms: str, limit: int, sources: Sequence[
     "Search for add-ons to install."
     from .prompts import PkgChoice, checkbox, confirm
 
-    assert ctx.obj
-    manager: managers.CliManager = ctx.obj.m
+    manager: _manager.Manager = ctx.obj.manager
 
-    entries = manager.run(manager.search(search_terms, limit, frozenset(sources) or None))
+    entries = run_with_progress(manager.search(search_terms, limit, frozenset(sources) or None))
     defns = [Defn(e.source, e.id) for e in entries]
     pkgs = (
         (d.with_(alias=r.slug), r)
-        for d, r in manager.run(manager.resolve(defns)).items()
+        for d, r in run_with_progress(manager.resolve(defns)).items()
         if models.is_pkg(r)
     )
-    choices = [PkgChoice(f'{p.name}  ({d.to_uri()}=={p.version})', d, pkg=p) for d, p in pkgs]
+    choices = [PkgChoice(f'{p.name}  ({d.to_urn()}=={p.version})', d, pkg=p) for d, p in pkgs]
     if choices:
         selections = checkbox('Select add-ons to install', choices=choices).unsafe_ask()
         if selections and confirm('Install selected add-ons?').unsafe_ask():
@@ -549,62 +635,69 @@ class ListFormats(str, Enum):
     show_default=True,
     help='Change the output format.',
 )
-@click.pass_obj
+@ManagerWrapper.pass_manager
 def list_installed(
-    obj: ManagerWrapper, addons: Sequence[Defn], output_format: ListFormats
+    manager: _manager.Manager, addons: Sequence[Defn], output_format: ListFormats
 ) -> None:
     "List installed add-ons."
-    from sqlalchemy import and_, or_
+    import sqlalchemy as sa
 
     def format_deps(pkg: models.Pkg):
         return (
-            (d.with_(alias=p.slug) if p else d).to_uri()
+            Defn(pkg.source, s or e.id).to_urn()
             for e in pkg.deps
-            for d in (Defn(pkg.source, e.id),)
-            for p in (obj.m.get_pkg(d),)
+            for s in (
+                manager.database.execute(
+                    sa.select(db.pkg.c.slug).filter_by(source=pkg.source, id=e.id)
+                ).scalar_one_or_none(),
+            )
         )
 
-    pkgs = (
-        obj.m.database.query(models.Pkg)
-        .filter(
-            or_(
-                *(
-                    models.Pkg.slug.contains(d.alias)
-                    if d.source == '*'
-                    else and_(
-                        models.Pkg.source == d.source,
-                        or_(models.Pkg.id == d.alias, models.Pkg.slug == d.alias),
+    pkgs = [
+        models.Pkg.from_row_mapping(manager.database, p)
+        for p in manager.database.execute(
+            sa.select(db.pkg)
+            .filter(
+                sa.or_(
+                    *(
+                        db.pkg.c.slug.contains(d.alias)
+                        if d.source == '*'
+                        else (db.pkg.c.source == d.source)
+                        & ((db.pkg.c.id == d.alias) | (db.pkg.c.slug == d.alias))
+                        for d in addons
                     )
-                    for d in addons
                 )
+                if addons
+                else True
             )
-            if addons
-            else True
+            .order_by(db.pkg.c.source, db.pkg.c.name)
         )
-        .order_by(models.Pkg.source, models.Pkg.name)
+        .mappings()
         .all()
-    )
+    ]
     if output_format is ListFormats.json:
-        click.echo(MultiPkgModel.parse_obj(pkgs).json(indent=2))
+        click.echo(models.PkgList.parse_obj(pkgs).json(indent=2))
+
     elif output_format is ListFormats.detailed:
         formatter = click.HelpFormatter(max_width=99)
         for pkg in pkgs:
-            with formatter.section(Defn.from_pkg(pkg).to_uri()):
+            with formatter.section(Defn.from_pkg(pkg).to_urn()):
                 formatter.write_dl(
-                    (
-                        ('Name', pkg.name),
-                        ('Description', textwrap.shorten(pkg.description, 280)),
-                        ('URL', pkg.url),
-                        ('Version', pkg.version),
-                        ('Date published', pkg.date_published.isoformat(' ', 'minutes')),
-                        ('Folders', ', '.join(f.name for f in pkg.folders)),
-                        ('Dependencies', ', '.join(format_deps(pkg))),
-                        ('Options', f'{{"strategy": "{pkg.options.strategy}"}}'),
-                    )
+                    [
+                        ('name', pkg.name),
+                        ('description', textwrap.shorten(pkg.description, 280)),
+                        ('url', pkg.url),
+                        ('version', pkg.version),
+                        ('date_published', pkg.date_published.isoformat(' ', 'minutes')[:-6]),
+                        ('folders', ', '.join(f.name for f in pkg.folders)),
+                        ('deps', ', '.join(format_deps(pkg))),
+                        ('options.strategy', pkg.options.strategy),
+                    ]
                 )
         click.echo(formatter.getvalue(), nl=False)
+
     else:
-        click.echo(''.join(f'{Defn.from_pkg(p).to_uri()}\n' for p in pkgs), nl=False)
+        click.echo(''.join(f'{Defn.from_pkg(p).to_urn()}\n' for p in pkgs), nl=False)
 
 
 @main.command(hidden=True)
@@ -617,32 +710,84 @@ def info(ctx: click.Context, addon: Defn) -> None:
 
 @main.command()
 @click.argument('addon', callback=_with_manager(partial(parse_into_defn, raise_invalid=False)))
-@click.pass_obj
-def reveal(obj: ManagerWrapper, addon: Defn) -> None:
+@ManagerWrapper.pass_manager
+def reveal(manager: _manager.Manager, addon: Defn) -> None:
     "Bring an add-on up in your file manager."
-    pkg = obj.m.get_pkg(addon, partial_match=True)
+    pkg = manager.get_pkg(addon, partial_match=True)
     if pkg:
-        click.launch(str(obj.m.config.addon_dir / pkg.folders[0].name), locate=True)
+        click.launch(str(manager.config.addon_dir / pkg.folders[0].name), locate=True)
     else:
         Report([(addon, R.PkgNotInstalled())]).generate_and_exit()
 
 
 @main.command()
-@click.argument('addon', callback=_with_manager(partial(parse_into_defn, raise_invalid=False)))
-@click.pass_obj
-def view_changelog(obj: ManagerWrapper, addon: Defn) -> None:
+@click.argument(
+    'addon', callback=_with_manager(partial(parse_into_defn, raise_invalid=False)), required=False
+)
+@click.option(
+    '--convert',
+    is_flag=True,
+    default=False,
+    help='Convert output to plain text.  Requires pandoc.',
+)
+@ManagerWrapper.pass_manager
+def view_changelog(manager: _manager.Manager, addon: Defn | None, convert: bool) -> None:
     "View the changelog of an installed add-on."
-    pkg = obj.m.get_pkg(addon, partial_match=True)
-    if pkg:
-        click.echo_via_pager(obj.m.run(obj.m.get_changelog(pkg.changelog_url)))
+
+    def do_convert(source: str, changelog: str):
+        import subprocess
+
+        changelog_format = manager.resolvers[source].changelog_format
+        if changelog_format not in {ChangelogFormat.html, ChangelogFormat.markdown}:
+            return changelog
+        else:
+            return subprocess.check_output(
+                ['pandoc', '-f', changelog_format, '-t', 'plain'], input=changelog, text=True
+            )
+
+    if addon:
+        pkg = manager.get_pkg(addon, partial_match=True)
+        if pkg:
+            changelog = run_with_progress(manager.get_changelog(pkg.changelog_url))
+            if convert:
+                changelog = do_convert(pkg.source, changelog)
+            click.echo_via_pager(changelog)
+        else:
+            Report([(addon, R.PkgNotInstalled())]).generate_and_exit()
+
     else:
-        Report([(addon, R.PkgNotInstalled())]).generate_and_exit()
+        import sqlalchemy as sa
+
+        last_installed_changelog_urls = manager.database.execute(
+            sa.select(db.pkg.c.source, db.pkg.c.slug, db.pkg.c.changelog_url)
+            .join(db.pkg_version_log)
+            .filter(
+                db.pkg_version_log.c.install_time
+                >= sa.select(
+                    sa.func.datetime(sa.func.max(db.pkg_version_log.c.install_time), '-1 minute')
+                )
+                .join(db.pkg)
+                .scalar_subquery()
+            )
+        ).all()
+        changelogs = run_with_progress(
+            gather(manager.get_changelog(m.changelog_url) for m in last_installed_changelog_urls)
+        )
+        if convert:
+            changelogs = (
+                do_convert(m.source, c) for m, c in zip(last_installed_changelog_urls, changelogs)
+            )
+        click.echo_via_pager(
+            '\n\n'.join(
+                Defn(m.source, m.slug).to_urn() + ':\n' + textwrap.indent(c, '  ')
+                for m, c in zip(last_installed_changelog_urls, changelogs)
+            )
+        )
 
 
 def _show_active_config(ctx: click.Context, __: click.Parameter, value: bool) -> None:
     if value:
-        assert ctx.obj
-        click.echo(ctx.obj.m.config.json(indent=2))
+        click.echo(ctx.obj.manager.config.json(indent=2))
         ctx.exit()
 
 
@@ -667,7 +812,7 @@ def _show_active_config(ctx: click.Context, __: click.Parameter, value: bool) ->
 def configure(ctx: click.Context, promptless: bool) -> Config:
     "Configure instawow."
     if promptless:
-        constructor = Config
+        make_config = Config
     else:
         from .prompts import PydanticValidator, path, select
 
@@ -681,9 +826,9 @@ def configure(ctx: click.Context, promptless: bool) -> Config:
             choices=list(Flavour),
             initial_choice=Config.infer_flavour(addon_dir),
         ).unsafe_ask()
-        constructor = partial(Config, addon_dir=addon_dir, game_flavour=game_flavour)
+        make_config = partial(Config, addon_dir=addon_dir, game_flavour=game_flavour)
 
-    config = constructor(profile=ctx.find_root().params['profile']).write()
+    config = make_config(profile=ctx.find_root().params['profile']).write()
     click.echo(f'Configuration written to: {config.config_file}')
     return config
 
@@ -694,23 +839,23 @@ def _weakauras_group() -> None:
 
 
 @_weakauras_group.command('build')
-@click.pass_obj
-def build_weakauras_companion(obj: ManagerWrapper) -> None:
+@ManagerWrapper.pass_manager
+def build_weakauras_companion(manager: _manager.Manager) -> None:
     "Build the WeakAuras Companion add-on."
     from .wa_updater import BuilderConfig, WaCompanionBuilder
 
     config = BuilderConfig()
-    obj.m.run(WaCompanionBuilder(obj.m, config).build())
+    run_with_progress(WaCompanionBuilder(manager, config).build())
 
 
 @_weakauras_group.command('list')
-@click.pass_obj
-def list_installed_wago_auras(obj: ManagerWrapper) -> None:
+@ManagerWrapper.pass_manager
+def list_installed_wago_auras(manager: _manager.Manager) -> None:
     "List WeakAuras installed from Wago."
     from .wa_updater import BuilderConfig, WaCompanionBuilder
 
     config = BuilderConfig()
-    aura_groups = WaCompanionBuilder(obj.m, config).extract_installed_auras()
+    aura_groups = WaCompanionBuilder(manager, config).extract_installed_auras()
     installed_auras = sorted(
         (g.filename, textwrap.fill(a.id, width=30, max_lines=1), a.url)
         for g in aura_groups
@@ -731,8 +876,6 @@ def _parse_datetime(_: click.Context, __: click.Parameter, value: str | None) ->
 @click.option('--age-cutoff', callback=_parse_datetime)
 def generate_catalogue(filename: str, age_cutoff: datetime | None) -> None:
     "Generate the master catalogue."
-    import asyncio
-
     from .resolvers import Catalogue
 
     catalogue = asyncio.run(Catalogue.collate(age_cutoff))
@@ -747,22 +890,14 @@ def generate_catalogue(filename: str, age_cutoff: datetime | None) -> None:
     )
 
 
-@main.command(hidden=True)
-@click.option(
-    '--log-to-stderr', is_flag=True, default=False, help="Output log to stderr for debugging."
-)
+@main.command(hidden=importlib.util.find_spec('instawow_gui') is None)
 @click.pass_context
-def gui(ctx: click.Context, log_to_stderr: bool) -> None:
+def gui(ctx: click.Context) -> None:
     "Fire up the GUI."
-    from instawow_gui import InstawowApp
+    from instawow_gui.app import InstawowApp
 
-    log_level = ctx.find_root().params['log_level']
-    _set_mac_multiprocessing_start_method()
-    _override_asyncio_loop_policy()
-    _set_asyncio_debug(log_level == 'DEBUG')
-
-    if not log_to_stderr:
-        dummy_config = Config.get_dummy_config(profile='__jsonrpc__').ensure_dirs()
-        setup_logging(dummy_config, log_level)
+    dummy_config = Config.get_dummy_config(profile='__jsonrpc__').ensure_dirs()
+    params = ctx.find_root().params
+    setup_logging(dummy_config, params['log_level'], params['log_to_stderr'])
 
     InstawowApp(version=__version__).main_loop()
