@@ -61,9 +61,8 @@ class Flavour(str, Enum):
     # will inherit the "_classic_" folder.  This way we won't have to
     # perform a migration or require user intervention.
     retail = 'retail'
-    classic = 'classic'
     vanilla_classic = 'vanilla_classic'
-    burning_crusade_classic = classic
+    burning_crusade_classic = 'classic'
 
 
 class Config(BaseConfig):
@@ -92,10 +91,10 @@ class Config(BaseConfig):
         tail = tuple(map(str.casefold, PurePath(folder).parts[-3:]))
         if len(tail) != 3 or tail[1:] != ('interface', 'addons'):
             return Flavour.retail
-        elif tail[0] == '_classic_era_':
+        elif tail[0] in {'_classic_era_', '_classic_era_ptr_'}:
             return Flavour.vanilla_classic
         elif tail[0] in {'_classic_', '_classic_beta_', '_classic_ptr_'}:
-            return Flavour.classic
+            return Flavour.burning_crusade_classic
         else:
             return Flavour.retail
 
@@ -126,14 +125,14 @@ class Config(BaseConfig):
 
     def ensure_dirs(self) -> Config:
         "Create the various folders used by instawow."
-        for dir_ in (
+        for dir_ in [
             self.config_dir,
             self.profile_dir,
             self.logging_dir,
             self.plugin_dir,
             self.temp_dir,
             self.cache_dir,
-        ):
+        ]:
             dir_.mkdir(exist_ok=True, parents=True)
         return self
 
@@ -179,10 +178,14 @@ class Config(BaseConfig):
         return self.temp_dir / 'cache'
 
 
-def setup_logging(config: Config, log_level: str = 'INFO') -> int:
+def setup_logging(
+    config: Config, log_level: str = 'INFO', log_to_stderr: bool = False
+) -> int | None:
     import logging
 
-    class InterceptHandler(logging.Handler):
+    class InterceptHandler(logging.Handler):  # pragma: no cover
+        logging_filename = getattr(logging, '__file__', None)
+
         def emit(self, record: logging.LogRecord) -> None:
             # Get the corresponding Loguru level if it exists
             try:
@@ -193,7 +196,7 @@ def setup_logging(config: Config, log_level: str = 'INFO') -> int:
             # Find caller from where the logged message originated
             frame = logging.currentframe()
             depth = 2
-            while frame and frame.f_code.co_filename == getattr(logging, '__file__', None):
+            while frame and frame.f_code.co_filename == self.logging_filename:
                 frame = frame.f_back
                 depth += 1
 
@@ -201,12 +204,16 @@ def setup_logging(config: Config, log_level: str = 'INFO') -> int:
 
     logging.basicConfig(handlers=[InterceptHandler()], level=log_level)
 
-    handler = {
-        'sink': config.logging_dir / 'error.log',
-        'level': log_level,
-        'rotation': '5 MB',
-        'retention': 5,  # Number of log files to keep
-        'enqueue': True,
-    }
-    (handler_id,) = logger.configure(handlers=(handler,))
-    return handler_id
+    if not log_to_stderr:
+        (handler_id,) = logger.configure(
+            handlers=[
+                {
+                    'sink': config.logging_dir / 'error.log',
+                    'level': log_level,
+                    'rotation': '5 MB',
+                    'retention': 5,  # Number of log files to keep
+                    'enqueue': True,
+                },
+            ]
+        )
+        return handler_id
