@@ -19,22 +19,23 @@ import json
 from pathlib import Path, PurePath
 from shutil import copy
 from tempfile import NamedTemporaryFile
-from types import SimpleNamespace
 from typing import TYPE_CHECKING, Any, TypeVar
 import urllib.parse
 
 from loguru import logger
-import sqlalchemy
-import sqlalchemy.exc
-import sqlalchemy.orm
+import sqlalchemy as sa
+import sqlalchemy.exc as sa_exc
+import sqlalchemy.future as sa_future
 from typing_extensions import Literal, TypeAlias, TypedDict
 from yarl import URL
 
-from . import _deferred_types, results as R
+from . import _deferred_types, db, models
+from . import results as R
+from .common import Strategy
 from .config import Config
-from .models import Pkg, PkgFolder, PkgVersionLog, is_pkg
 from .plugins import load_plugins
 from .resolvers import (
+    BaseResolver,
     Catalogue,
     CatalogueEntry,
     CurseResolver,
@@ -42,30 +43,24 @@ from .resolvers import (
     GithubResolver,
     InstawowResolver,
     Resolver,
-    Strategy,
-    TownlongYakResolver,
     TukuiResolver,
     WowiResolver,
-    normalise_names,
 )
 from .utils import (
     bucketise,
     chain_dict,
-    copy_resources,
     file_uri_to_path,
     find_zip_base_dirs,
     gather,
     is_not_stale,
-    make_progress_bar,
     make_zip_member_filter,
     move,
-    run_in_thread as t,
-    shasum,
-    trash,
-    uniq,
+    normalise_names,
 )
+from .utils import run_in_thread as t
+from .utils import shasum, trash, uniq
 
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     _BaseResolverDict: TypeAlias = 'dict[str, Resolver]'
 else:
     _BaseResolverDict = dict
@@ -76,7 +71,7 @@ _C = TypeVar('_C', bound='Callable[..., Awaitable[object]]')
 
 USER_AGENT = 'instawow (https://github.com/layday/instawow)'
 
-DB_REVISION = '764fa963cc71'
+DB_REVISION = '9b8df9661cdb'
 
 
 class _GenericDownloadTraceRequestCtx(TypedDict):
@@ -87,7 +82,7 @@ class _GenericDownloadTraceRequestCtx(TypedDict):
 class _PkgDownloadTraceRequestCtx(TypedDict):
     report_progress: Literal['pkg_download']
     manager: Manager
-    pkg: Pkg
+    pkg: models.Pkg
 
 
 TraceRequestCtx: TypeAlias = '_GenericDownloadTraceRequestCtx | _PkgDownloadTraceRequestCtx | None'
@@ -113,7 +108,7 @@ async def _open_temp_writer() -> AsyncIterator[tuple[Path, Callable[[bytes], Awa
 
 
 @contextmanager
-def _open_archive(path: PurePath) -> Iterator[tuple[set[str], Callable[[Path], None]]]:
+def _open_pkg_archive(path: PurePath) -> Iterator[tuple[set[str], Callable[[Path], None]]]:
     from zipfile import ZipFile
 
     ZIP_EXCLUDES = {
@@ -132,7 +127,9 @@ def _open_archive(path: PurePath) -> Iterator[tuple[set[str], Callable[[Path], N
         yield (base_dirs, extract)
 
 
-async def _download_archive(manager: Manager, pkg: Pkg, *, chunk_size: int = 4096) -> Path:
+async def _download_pkg_archive(
+    manager: Manager, pkg: models.Pkg, *, chunk_size: int = 4096
+) -> Path:
     url = pkg.download_url
     dest = manager.config.cache_dir / shasum(
         pkg.source, pkg.id, pkg.version, manager.config.game_flavour
@@ -148,9 +145,6 @@ async def _download_archive(manager: Manager, pkg: Pkg, *, chunk_size: int = 409
             trace_request_ctx=_PkgDownloadTraceRequestCtx(
                 report_progress='pkg_download', manager=manager, pkg=pkg
             ),
-            # This is needed for Townlong Yak.
-            # See: https://github.com/aio-libs/aiohttp/issues/3904#issuecomment-632661245
-            headers={'Connection': 'keep-alive'},
         ) as response, _open_temp_writer() as (temp_path, write):
             async for chunk in response.content.iter_chunked(chunk_size):
                 await write(chunk)
@@ -185,7 +179,7 @@ async def cache_response(
 
     dest = manager.config.cache_dir / shasum(url, request_extra)
     if await t(is_not_stale)(dest, ttl):
-        logger.debug(f'{url} is cache at {dest} (ttl: {ttl})')
+        logger.debug(f'{url} is cached at {dest} (ttl: {ttl})')
         text = await t(dest.read_text)(encoding='utf-8')
     else:
         text = await make_request()
@@ -194,87 +188,86 @@ async def cache_response(
     return json.loads(text) if is_json else text
 
 
-def _should_migrate(engine: Any) -> bool:
-    """Check if the database version is the same as ``DB_REVISION``;
-    if not, a migration is required.
+def prepare_database(config: Config) -> sa_future.Engine:
+    engine = sa.create_engine(
+        f'sqlite:///{config.db_file}',
+        # We wanna be able to operate on SQLite objects from executor threads
+        # for convenience, when performing disk I/O
+        connect_args={'check_same_thread': False},
+        # echo=True,
+        future=True,
+    )
 
-    Importing Alembic is prohibitively expensive in the CLI
-    (adds about 250 ms to start-up time on my MBP) so we defer
-    to SQLAlchemy.
-    """
-    with engine.begin() as connection:
+    with engine.connect() as connection:
         try:
             current = connection.execute(
-                sqlalchemy.text(
+                sa.text(
                     'SELECT version_num FROM alembic_version WHERE version_num = :version_num'
                 ),
                 {'version_num': DB_REVISION},
             ).scalar()
-        except sqlalchemy.exc.OperationalError:
-            return True
+        except sa_exc.OperationalError:
+            should_migrate = True
         else:
-            return not current
+            should_migrate = not current
 
+    if should_migrate:
+        import alembic.command
+        import alembic.config
 
-def prepare_database(config: Config) -> sqlalchemy.orm.Session:
-    from .models import ModelBase
+        alembic_config = alembic.config.Config()
+        alembic_config.set_main_option('script_location', f'{__package__}:migrations')
+        alembic_config.set_main_option('sqlalchemy.url', str(engine.url))
 
-    engine = sqlalchemy.create_engine(
-        f'sqlite:///{config.db_file}',
-        # We wanna be able to operate on SQLite objects from
-        # executor threads for convenience, when performing disk I/O
-        connect_args={'check_same_thread': False},
-    )
+        if sa.inspect(engine).get_table_names():
+            alembic.command.upgrade(alembic_config, DB_REVISION)
+        else:
+            db.metadata.create_all(engine)
+            alembic.command.stamp(alembic_config, DB_REVISION)
 
-    if _should_migrate(engine):
-        from alembic.command import stamp, upgrade
-        from alembic.config import Config as AlembicConfig
-
-        with copy_resources(
-            f'{__package__}.migrations',
-            f'{__package__}.migrations.versions',
-        ) as tmp_dir:
-            alembic_config = AlembicConfig()
-            alembic_config.set_main_option(
-                'script_location', str(tmp_dir / __package__ / 'migrations')
-            )
-            alembic_config.set_main_option('sqlalchemy.url', str(engine.url))
-
-            if sqlalchemy.inspect(engine).get_table_names():
-                upgrade(alembic_config, DB_REVISION)
-            else:
-                ModelBase.metadata.create_all(engine)
-                stamp(alembic_config, DB_REVISION)
-
-    return sqlalchemy.orm.sessionmaker(bind=engine)()
+    return engine
 
 
 def init_web_client(**kwargs: Any) -> _deferred_types.aiohttp.ClientSession:
     from aiohttp import ClientSession, ClientTimeout, TCPConnector
 
+    try:
+        import certifi
+    except ModuleNotFoundError:
+        connector = TCPConnector(force_close=True, limit_per_host=10)
+    else:
+        from importlib.resources import read_text
+        import ssl
+
+        logger.info('loading certifi certs')
+        ssl_context = ssl.create_default_context(
+            cadata=read_text(certifi, 'cacert.pem', encoding='ascii')
+        )
+        connector = TCPConnector(force_close=True, limit_per_host=10, ssl=ssl_context)
+
     kwargs = {
-        'connector': TCPConnector(force_close=True, limit_per_host=10),
+        'connector': connector,
         'headers': {'User-Agent': USER_AGENT},
         'trust_env': True,  # Respect the 'http_proxy' env var
-        'timeout': ClientTimeout(connect=10, sock_read=10),  # type: ignore
+        'timeout': ClientTimeout(connect=60, sock_connect=10, sock_read=10),
         **kwargs,
     }
     return ClientSession(**kwargs)
 
 
 @object.__new__
-class _DummyResolver(Resolver):
-    strategies = set()
+class _DummyResolver(BaseResolver):
+    strategies = frozenset()
 
     async def resolve(
         self, defns: Sequence[Defn]
-    ) -> dict[Defn, Pkg | R.ManagerError | R.InternalError]:
+    ) -> dict[Defn, models.Pkg | R.ManagerError | R.InternalError]:
         return dict.fromkeys(defns, R.PkgSourceInvalid())
 
 
 class _ResolverDict(_BaseResolverDict):
     def __missing__(self, key: str) -> Resolver:
-        return _DummyResolver
+        return _DummyResolver  # type: ignore
 
 
 async def capture_manager_exc_async(
@@ -332,22 +325,23 @@ class Manager:
         TukuiResolver,
         GithubResolver,
         InstawowResolver,
-        TownlongYakResolver,
     )
 
     def __init__(
         self,
         config: Config,
-        database: sqlalchemy.orm.Session,
+        database: sa_future.Connection,
     ) -> None:
-        self.config = config
-        self.database = database
+        self.config: Config = config
+        self.database: sa_future.Connection = database
 
         plugin_hook = load_plugins()
         resolver_classes = chain(
             (r for g in plugin_hook.instawow_add_resolvers() for r in g), self.RESOLVERS
         )
-        self.resolvers = _ResolverDict((r.source, r(self)) for r in resolver_classes)
+        self.resolvers: _ResolverDict = _ResolverDict(
+            (r.source, r(self)) for r in resolver_classes
+        )
 
         self._catalogue = None
 
@@ -364,8 +358,8 @@ class Manager:
             _locks.set(locks)
 
     @classmethod
-    def from_config(cls: type[_T], config: Config) -> _T:
-        return cls(config, prepare_database(config))
+    def from_config(cls, config: Config) -> Manager:
+        return cls(config, prepare_database(config).connect())
 
     @property
     def web_client(self) -> _deferred_types.aiohttp.ClientSession:
@@ -388,41 +382,62 @@ class Manager:
         aliases_from_url = (
             (r.source, a)
             for r in self.resolvers.values()
-            for a in (r.get_alias_from_url(value),)
+            for a in (r.get_alias_from_url(URL(value)),)
             if a
         )
         return next(chain(aliases_from_url, from_urn(), (None,)))
 
-    def get_pkg(self, defn: Defn, partial_match: bool = False) -> Pkg | None:
-        "Retrieve an installed package from a definition."
+    def check_pkg_exists(self, defn: Defn) -> bool:
         return (
-            (
-                self.database.query(Pkg)
+            self.database.execute(
+                sa.select(sa.func.count())
+                .select_from(db.pkg)
                 .filter(
-                    Pkg.source == defn.source,
-                    (Pkg.id == defn.alias) | (Pkg.slug == defn.alias) | (Pkg.id == defn.id),
+                    db.pkg.c.source == defn.source,
+                    (db.pkg.c.id == defn.alias)
+                    | (db.pkg.c.id == defn.id)
+                    | (db.pkg.c.slug == defn.alias),
                 )
-                .first()
-            )
-            or partial_match
-            and (
-                self.database.query(Pkg)
-                .filter(Pkg.slug.contains(defn.alias))
-                .order_by(Pkg.name)
-                .first()
-            )
-            or None
+            ).scalar()
+            != 0
         )
 
-    def install_pkg(self, pkg: Pkg, archive: Path, replace: bool) -> R.PkgInstalled:
-        "Install a package."
-        with _open_archive(archive) as (top_level_folders, extract):
-            installed_conflicts: list[Pkg] = (
-                self.database.query(Pkg)
-                .join(Pkg.folders)
-                .filter(PkgFolder.name.in_(top_level_folders))
-                .all()
+    def get_pkg(self, defn: Defn, partial_match: bool = False) -> models.Pkg | None:
+        "Retrieve an installed package from a definition."
+        maybe_row_mapping = (
+            self.database.execute(
+                sa.select(db.pkg).filter(
+                    db.pkg.c.source == defn.source,
+                    (db.pkg.c.id == defn.alias)
+                    | (db.pkg.c.id == defn.id)
+                    | (db.pkg.c.slug == defn.alias),
+                )
             )
+            .mappings()
+            .one_or_none()
+        )
+        if maybe_row_mapping is None and partial_match:
+            maybe_row_mapping = (
+                self.database.execute(
+                    sa.select(db.pkg)
+                    .filter(db.pkg.c.slug.contains(defn.alias))
+                    .order_by(db.pkg.c.name)
+                )
+                .mappings()
+                .first()
+            )
+        if maybe_row_mapping is not None:
+            return models.Pkg.from_row_mapping(self.database, maybe_row_mapping)
+
+    def install_pkg(self, pkg: models.Pkg, archive: Path, replace: bool) -> R.PkgInstalled:
+        "Install a package."
+        with _open_pkg_archive(archive) as (top_level_folders, extract):
+            installed_conflicts = self.database.execute(
+                sa.select(db.pkg)
+                .distinct()
+                .join(db.pkg_folder)
+                .filter(db.pkg_folder.c.name.in_(top_level_folders))
+            ).all()
             if installed_conflicts:
                 raise R.PkgConflictsWithInstalled(installed_conflicts)
 
@@ -440,53 +455,50 @@ class Manager:
                     raise R.PkgConflictsWithUnreconciled(unreconciled_conflicts)
 
             extract(self.config.addon_dir)
-            pkg.folders = [PkgFolder(name=f) for f in sorted(top_level_folders)]
 
-        self.database.add(pkg)
-        self.database.merge(
-            PkgVersionLog(version=pkg.version, pkg_source=pkg.source, pkg_id=pkg.id)
+        pkg = models.Pkg.parse_obj(
+            {**pkg.__dict__, 'folders': [{'name': f} for f in sorted(top_level_folders)]}
         )
-        self.database.commit()
-
+        pkg.insert(self.database)
         return R.PkgInstalled(pkg)
 
-    def update_pkg(self, old_pkg: Pkg, new_pkg: Pkg, archive: Path) -> R.PkgUpdated:
+    def update_pkg(self, pkg1: models.Pkg, pkg2: models.Pkg, archive: Path) -> R.PkgUpdated:
         "Update a package."
-        with _open_archive(archive) as (top_level_folders, extract):
-            installed_conflicts: list[Pkg] = (
-                self.database.query(Pkg)
-                .join(Pkg.folders)
-                .filter(PkgFolder.pkg_source != new_pkg.source, PkgFolder.pkg_id != new_pkg.id)
-                .filter(PkgFolder.name.in_(top_level_folders))
-                .all()
-            )
+        with _open_pkg_archive(archive) as (top_level_folders, extract):
+            installed_conflicts = self.database.execute(
+                sa.select(db.pkg)
+                .distinct()
+                .join(db.pkg_folder)
+                .filter(
+                    db.pkg_folder.c.pkg_source != pkg2.source,
+                    db.pkg_folder.c.pkg_id != pkg2.id,
+                    db.pkg_folder.c.name.in_(top_level_folders),
+                )
+            ).all()
             if installed_conflicts:
                 raise R.PkgConflictsWithInstalled(installed_conflicts)
 
-            unreconciled_conflicts = top_level_folders - {f.name for f in old_pkg.folders} & {
+            unreconciled_conflicts = top_level_folders - {f.name for f in pkg1.folders} & {
                 f.name for f in self.config.addon_dir.iterdir()
             }
             if unreconciled_conflicts:
                 raise R.PkgConflictsWithUnreconciled(unreconciled_conflicts)
 
             trash(
-                [self.config.addon_dir / f.name for f in old_pkg.folders],
+                [self.config.addon_dir / f.name for f in pkg1.folders],
                 dest=self.config.temp_dir,
                 missing_ok=True,
             )
             extract(self.config.addon_dir)
-            new_pkg.folders = [PkgFolder(name=f) for f in sorted(top_level_folders)]
 
-        self.database.delete(old_pkg)
-        self.database.add(new_pkg)
-        self.database.merge(
-            PkgVersionLog(version=new_pkg.version, pkg_source=new_pkg.source, pkg_id=new_pkg.id)
+        pkg2 = models.Pkg.parse_obj(
+            {**pkg2.__dict__, 'folders': [{'name': f} for f in sorted(top_level_folders)]}
         )
-        self.database.commit()
+        pkg1.delete(self.database)
+        pkg2.insert(self.database)
+        return R.PkgUpdated(pkg1, pkg2)
 
-        return R.PkgUpdated(old_pkg, new_pkg)
-
-    def remove_pkg(self, pkg: Pkg, keep_folders: bool) -> R.PkgRemoved:
+    def remove_pkg(self, pkg: models.Pkg, keep_folders: bool) -> R.PkgRemoved:
         "Remove a package."
         if not keep_folders:
             trash(
@@ -494,9 +506,8 @@ class Manager:
                 dest=self.config.temp_dir,
                 missing_ok=True,
             )
-        self.database.delete(pkg)
-        self.database.commit()
 
+        pkg.delete(self.database)
         return R.PkgRemoved(pkg)
 
     @_with_lock('load catalogue', False)
@@ -506,8 +517,8 @@ class Manager:
             label = 'Synchronising catalogue'
             url = (
                 'https://raw.githubusercontent.com/layday/instawow-data/data/'
-                'master-catalogue-v3.compact.json'
-            )  # v3
+                'master-catalogue-v4.compact.json'
+            )  # v4
             raw_catalogue = await cache_response(self, url, {'hours': 4}, label=label)
             self._catalogue = Catalogue.parse_obj(raw_catalogue)
         return self._catalogue
@@ -520,7 +531,7 @@ class Manager:
         complexity for something that I would never expect to
         encounter in the wild.
         """
-        pkgs = [r for r in results if is_pkg(r)]
+        pkgs = [r for r in results if models.is_pkg(r)]
         dep_defns = uniq(
             filterfalse(
                 {(p.source, p.id) for p in pkgs}.__contains__,
@@ -531,7 +542,9 @@ class Manager:
             return {}
 
         deps = await self.resolve(list(starmap(Defn, dep_defns)))
-        pretty_deps = {d.with_(alias=r.slug) if is_pkg(r) else d: r for d, r in deps.items()}
+        pretty_deps = {
+            d.with_(alias=r.slug) if models.is_pkg(r) else d: r for d, r in deps.items()
+        }
         return pretty_deps
 
     async def resolve(self, defns: Sequence[Defn], with_deps: bool = False) -> dict[Defn, Any]:
@@ -580,14 +593,12 @@ class Manager:
         sources: Set[str] | None = None,
     ) -> list[CatalogueEntry]:
         "Search the master catalogue for packages by name."
-        import heapq
-
-        from jellyfish import jaro_winkler_similarity
+        import rapidfuzz
 
         catalogue = await self.synchronise()
 
         w = 0.5  # Weighing edit distance and download score equally
-        normalise = normalise_names()
+        normalise = normalise_names('')
 
         if sources is None:
             sources = self.resolvers.keys()
@@ -595,21 +606,19 @@ class Manager:
         s = normalise(search_terms)
         tokens_to_entries = bucketise(
             (
-                (i.normalised_name, i)
+                (normalise(i.name), i)
                 for i in catalogue.__root__
                 if self.config.game_flavour in i.game_flavours and i.source in sources
             ),
             key=lambda v: v[0],
         )
-        matches = heapq.nlargest(
-            limit,
-            ((jaro_winkler_similarity(s, n), n) for n in tokens_to_entries),
-            key=lambda v: v[0],
+        matches = rapidfuzz.process.extract(
+            s, list(tokens_to_entries), scorer=rapidfuzz.fuzz.WRatio, limit=limit
         )
         weighted_entries = sorted(
             (
-                (-(s * w + i.derived_download_score * (1 - w)), i)
-                for s, m in matches
+                (-((s / 100) * w + i.derived_download_score * (1 - w)), i)
+                for m, s, _ in matches
                 for _, i in tokens_to_entries[m]
             ),
             key=lambda v: v[0],
@@ -621,40 +630,43 @@ class Manager:
         self, defns: Sequence[Defn], replace: bool
     ) -> dict[Defn, R.PkgInstalled | R.ManagerError | R.InternalError]:
         "Install packages from a definition list."
-        # We'll weed out installed dependencies from results after resolving.
-        # Doing it this way isn't particularly efficient but avoids having to
-        # deal with local state in resolvers.
+        # We'll weed out installed deps from the results after resolving -
+        # doing it this way isn't particularly efficient but avoids having to
+        # deal with local state in `resolve()`
         resolve_results = await self.resolve(
-            list(compress(defns, (not self.get_pkg(d) for d in defns))),
+            list(compress(defns, (not self.check_pkg_exists(d) for d in defns))),
             with_deps=True,
         )
         resolve_results = dict(
-            compress(resolve_results.items(), (not self.get_pkg(d) for d in resolve_results))
+            compress(
+                resolve_results.items(), (not self.check_pkg_exists(d) for d in resolve_results)
+            )
         )
-        installables = {d: r for d, r in resolve_results.items() if is_pkg(r)}
+        installables = {d: r for d, r in resolve_results.items() if models.is_pkg(r)}
         archives = await gather(
-            (_download_archive(self, r) for r in installables.values()), capture_manager_exc_async
+            (_download_pkg_archive(self, r) for r in installables.values()),
+            capture_manager_exc_async,
         )
         results = chain_dict(
             defns,
             R.PkgAlreadyInstalled(),
             resolve_results.items(),
-            ((d, a) for d, a in zip(installables, archives) if not isinstance(a, PurePath)),
             [
                 (d, await capture_manager_exc_async(t(self.install_pkg)(p, a, replace)))
-                for (d, p), a in zip(installables.items(), archives)
                 if isinstance(a, PurePath)
+                else (d, a)
+                for (d, p), a in zip(installables.items(), archives)
             ],
         )
         return results
 
     @_with_lock('change state')
     async def update(
-        self, defns: Sequence[Defn], retain_strategy: bool
+        self, defns: Sequence[Defn], retain_defn_strategy: bool
     ) -> dict[Defn, R.PkgUpdated | R.ManagerError | R.InternalError]:
         """Update installed packages from a definition list.
 
-        A ``retain_strategy`` value of false will instruct ``update``
+        A ``retain_defn_strategy`` value of false will instruct ``update``
         to extract the strategy from the installed package; otherwise
         the ``Defn`` strategy will be used.
         """
@@ -664,14 +676,14 @@ class Manager:
             # corresponding installed package.  Using the ID has the benefit
             # of resolving installed-but-renamed packages - the slug is
             # transient but the ID isn't
-            d.with_(id=p.id) if retain_strategy else Defn.from_pkg(p): d
+            d.with_(id=p.id) if retain_defn_strategy else Defn.from_pkg(p): d
             for d, p in defns_to_pkgs.items()
         }
         # Discard the reconstructed ``Defn``s
         resolve_results = {
             resolve_defns[d]: r for d, r in (await self.resolve(list(resolve_defns))).items()
         }
-        installables = {d: r for d, r in resolve_results.items() if is_pkg(r)}
+        installables = {d: r for d, r in resolve_results.items() if models.is_pkg(r)}
         updatables = {
             d: (o, n)
             for d, n in installables.items()
@@ -679,7 +691,8 @@ class Manager:
             if n.version != o.version
         }
         archives = await gather(
-            (_download_archive(self, n) for _, n in updatables.values()), capture_manager_exc_async
+            (_download_pkg_archive(self, n) for _, n in updatables.values()),
+            capture_manager_exc_async,
         )
         results = chain_dict(
             defns,
@@ -689,11 +702,11 @@ class Manager:
                 (d, R.PkgUpToDate(is_pinned=p.options.strategy == Strategy.version))
                 for d, p in installables.items()
             ),
-            ((d, a) for d, a in zip(updatables, archives) if not isinstance(a, PurePath)),
             [
                 (d, await capture_manager_exc_async(t(self.update_pkg)(o, n, a)))
-                for (d, (o, n)), a in zip(updatables.items(), archives)
                 if isinstance(a, PurePath)
+                else (d, a)
+                for (d, (o, n)), a in zip(updatables.items(), archives)
             ],
         )
         return results
@@ -739,97 +752,28 @@ class Manager:
         strategies = frozenset({Strategy.default, Strategy.version})
 
         def pin(
-            defn: Defn, pkg: Pkg | None
+            defn: Defn, pkg: models.Pkg | None
         ) -> R.PkgNotInstalled | R.PkgInstalled | R.PkgStrategyUnsupported:
             if not pkg:
                 return R.PkgNotInstalled()
+
             elif {defn.strategy} <= strategies <= self.resolvers[pkg.source].strategies:
-                pkg.options.strategy = defn.strategy
+                self.database.execute(
+                    sa.update(db.pkg_options)
+                    .filter_by(pkg_source=pkg.source, pkg_id=pkg.id)
+                    .values(strategy=defn.strategy)
+                )
                 self.database.commit()
-                return R.PkgInstalled(pkg)
+                row_mapping = (
+                    self.database.execute(
+                        sa.select(db.pkg).filter_by(source=pkg.source, id=pkg.id)
+                    )
+                    .mappings()
+                    .one()
+                )
+                return R.PkgInstalled(models.Pkg.from_row_mapping(self.database, row_mapping))
+
             else:
                 return R.PkgStrategyUnsupported(Strategy.version)
 
         return {d: await capture_manager_exc_async(t(pin)(d, self.get_pkg(d))) for d in defns}
-
-
-def _extract_filename_from_hdr(response: _deferred_types.aiohttp.ClientResponse) -> str:
-    from cgi import parse_header
-
-    from aiohttp import hdrs
-
-    _, cd_params = parse_header(response.headers.get(hdrs.CONTENT_DISPOSITION, ''))
-    filename = cd_params.get('filename') or response.url.name
-    return filename
-
-
-def _init_cli_web_client(
-    bar: _deferred_types.prompt_toolkit.shortcuts.ProgressBar, tickers: set[asyncio.Task[None]]
-) -> _deferred_types.aiohttp.ClientSession:
-    from aiohttp import TraceConfig, TraceRequestEndParams, hdrs
-
-    TICK_INTERVAL = 0.1
-
-    async def do_on_request_end(
-        client_session: _deferred_types.aiohttp.ClientSession,
-        trace_config_ctx: SimpleNamespace,
-        params: TraceRequestEndParams,
-    ) -> None:
-        trace_request_ctx: TraceRequestCtx = trace_config_ctx.trace_request_ctx
-        if trace_request_ctx:
-            response = params.response
-            label = (
-                trace_request_ctx.get('label')
-                or f'Downloading {_extract_filename_from_hdr(response)}'
-            )
-            total = response.content_length
-            if hdrs.CONTENT_ENCODING in response.headers:
-                # The encoded size is not exposed in the aiohttp streaming API.
-                # If the payload is encoded, ``total`` is set to ``None``
-                # for the progress bar to be rendered as indeterminate.
-                total = None
-
-            async def ticker() -> None:
-                counter = bar(label=label, total=total)
-                try:
-                    while not response.content.is_eof():
-                        counter.items_completed = response.content.total_bytes
-                        bar.invalidate()
-                        await asyncio.sleep(TICK_INTERVAL)
-                finally:
-                    bar.counters.remove(counter)
-
-            tickers.add(asyncio.create_task(ticker()))
-
-    trace_config = TraceConfig()
-    trace_config.on_request_end.append(do_on_request_end)
-    trace_config.freeze()
-    return init_web_client(trace_configs=[trace_config])
-
-
-@asynccontextmanager
-async def _cancel_tickers(tickers: set[asyncio.Task[None]]):
-    try:
-        yield
-    finally:
-        for ticker in tickers:
-            ticker.cancel()
-
-
-class CliManager(Manager):
-    def run(self, awaitable: Awaitable[_T]) -> _T:
-        with make_progress_bar() as bar:
-
-            async def run():
-                tickers: set[asyncio.Task[None]] = set()
-                async with _init_cli_web_client(bar, tickers) as web_client, _cancel_tickers(
-                    tickers
-                ):
-                    self.contextualise(web_client=web_client)
-                    return await awaitable
-
-            loop = asyncio.new_event_loop()
-            try:
-                return loop.run_until_complete(run())
-            finally:
-                loop.close()
