@@ -1,5 +1,6 @@
 from functools import partial
 import json
+from textwrap import dedent
 
 from click.testing import CliRunner
 from prompt_toolkit.application import create_app_session
@@ -10,7 +11,8 @@ import pytest
 from instawow import __version__
 from instawow.cli import main
 from instawow.config import Flavour
-from instawow.resolvers import MultiPkgModel
+from instawow.manager import Manager
+from instawow.models import PkgList
 
 
 @pytest.fixture
@@ -23,17 +25,13 @@ def feed_pt():
 
 @pytest.fixture
 def run(monkeypatch, event_loop, iw_config, iw_web_client):
-    def runner(self, awaitable):
-        self.contextualise(web_client=iw_web_client)
+    def runner(awaitable):
+        Manager.contextualise(web_client=iw_web_client)
         return event_loop.run_until_complete(awaitable)
 
-    monkeypatch.setattr('instawow.manager.CliManager.run', runner)
+    monkeypatch.setattr('instawow.cli.run_with_progress', runner)
     monkeypatch.setenv('INSTAWOW_CONFIG_DIR', str(iw_config.config_dir))
-    yield partial(
-        CliRunner().invoke,
-        main,
-        catch_exceptions=False,
-    )
+    yield partial(CliRunner().invoke, main, catch_exceptions=False)
 
 
 @pytest.fixture
@@ -264,57 +262,56 @@ def test_version_strategy_lifecycle(iw_config, run):
 
 
 def test_install_options(iw_config, run):
-    assert (
-        run(
-            'install'
-            ' curse:molinari'
-            ' -s latest curse:molinari'
-            ' --version 80000.57-Release curse:molinari'
-        ).output
-        == f'''\
-✓ curse:molinari
-  installed 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""}
-✗ curse:molinari
-  package folders conflict with installed package Molinari
-    (curse:20338)
-✗ curse:molinari
-  package folders conflict with installed package Molinari
-    (curse:20338)
-'''
+    assert run(
+        'install'
+        ' curse:molinari'
+        ' -s latest curse:molinari'
+        ' --version 80000.57-Release curse:molinari'
+    ).output == dedent(
+        f'''\
+        ✓ curse:molinari
+          installed 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""}
+        ✗ curse:molinari
+          package folders conflict with installed package Molinari
+            (curse:20338)
+        ✗ curse:molinari
+          package folders conflict with installed package Molinari
+            (curse:20338)
+        '''
     )
 
 
 def test_install_option_order_is_respected(run):
-    assert (
-        run(
-            'install'
-            ' --version 80000.57-Release curse:molinari'
-            ' -s latest curse:molinari'
-            ' curse:molinari'
-        ).output
-        == '''\
-✓ curse:molinari
-  installed 80000.57-Release
-✗ curse:molinari
-  package folders conflict with installed package Molinari
-    (curse:20338)
-✗ curse:molinari
-  package folders conflict with installed package Molinari
-    (curse:20338)
-'''
+    assert run(
+        'install'
+        ' --version 80000.57-Release curse:molinari'
+        ' -s latest curse:molinari'
+        ' curse:molinari'
+    ).output == dedent(
+        '''\
+        ✓ curse:molinari
+          installed 80000.57-Release
+        ✗ curse:molinari
+          package folders conflict with installed package Molinari
+            (curse:20338)
+        ✗ curse:molinari
+          package folders conflict with installed package Molinari
+            (curse:20338)
+        '''
     )
 
 
 def test_install_argument_is_not_required(iw_config, run):
-    assert (
-        run('install -s latest curse:molinari --version 80000.57-Release curse:molinari').output
-        == f'''\
-✓ curse:molinari
-  installed 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""}
-✗ curse:molinari
-  package folders conflict with installed package Molinari
-    (curse:20338)
-'''
+    assert run(
+        'install -s latest curse:molinari --version 80000.57-Release curse:molinari'
+    ).output == dedent(
+        f'''\
+        ✓ curse:molinari
+          installed 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""}
+        ✗ curse:molinari
+          package folders conflict with installed package Molinari
+            (curse:20338)
+        '''
     )
 
 
@@ -352,7 +349,7 @@ def test_rollback__unsupported(run, options):
     assert run('install wowi:13188-molinari').exit_code == 0
     assert (
         run(f'rollback {options} wowi:13188-molinari').output
-        == '✗ wowi:13188-molinari\n  source does not support rollback\n'
+        == "✗ wowi:13188-molinari\n  'version' strategy is not valid for source\n"
     )
 
 
@@ -369,9 +366,11 @@ def test_rollback__multiple_versions(feed_pt, iw_config, run):
     assert run('remove curse:molinari').exit_code == 0
     assert run('install curse:molinari').exit_code == 0
     feed_pt('\r\r')
-    assert (
-        run('rollback curse:molinari').output
-        == f'✓ curse:molinari\n  updated 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""} to 80000.57-Release\n'
+    assert run('rollback curse:molinari').output == dedent(
+        f'''\
+        ✓ curse:molinari
+          updated 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""} to 80000.57-Release
+        '''
     )
 
 
@@ -379,18 +378,25 @@ def test_rollback__multiple_versions_promptless(iw_config, run):
     assert run('install --version 80000.57-Release curse:molinari').exit_code == 0
     assert run('remove curse:molinari').exit_code == 0
     assert run('install curse:molinari').exit_code == 0
-    assert (
-        run('rollback --version 80000.57-Release curse:molinari').output
-        == f'✓ curse:molinari\n  updated 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""} to 80000.57-Release\n'
+    assert run('rollback --version 80000.57-Release curse:molinari').output == dedent(
+        f'''\
+        ✓ curse:molinari
+          updated 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""} to 80000.57-Release
+        '''
     )
 
 
 @pytest.mark.parametrize('options', ['', '--undo'])
 def test_rollback__rollback_multiple_versions(feed_pt, iw_config, run, options):
-    test_rollback__multiple_versions(feed_pt, iw_config, run)
-    assert (
-        run(f'rollback {options} curse:molinari').output
-        == f'✓ curse:molinari\n  updated 80000.57-Release to 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""}\n'
+    assert run('install curse:molinari').exit_code == 0
+    assert run('remove curse:molinari').exit_code == 0
+    assert run('install --version 80000.57-Release curse:molinari').exit_code == 0
+    feed_pt('\r\r')
+    assert run(f'rollback {options} curse:molinari').output == dedent(
+        f'''\
+        ✓ curse:molinari
+          updated 80000.57-Release to 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""}
+        '''
     )
 
 
@@ -411,9 +417,11 @@ def test_reconcile__list_unreconciled(faux_molinari_and_run):
 
 
 def test_reconcile__auto_reconcile(iw_config, faux_molinari_and_run):
-    assert (
-        faux_molinari_and_run('reconcile --auto').output
-        == f'✓ curse:molinari\n  installed 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""}\n'
+    assert faux_molinari_and_run('reconcile --auto').output == dedent(
+        f'''\
+        ✓ curse:molinari
+          installed 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""}
+        '''
     )
 
 
@@ -425,7 +433,12 @@ def test_reconcile__abort_interactive_reconciliation(feed_pt, faux_molinari_and_
 def test_reconcile__complete_interactive_reconciliation(feed_pt, iw_config, faux_molinari_and_run):
     feed_pt('\r\r')
     assert faux_molinari_and_run('reconcile').output.endswith(
-        f'✓ curse:molinari\n  installed 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""}\n'
+        dedent(
+            f'''\
+            ✓ curse:molinari
+              installed 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""}
+            '''
+        )
     )
 
 
@@ -447,34 +460,52 @@ def test_search__exit_after_selection(feed_pt, run):
     assert run('search molinari').output == ''
 
 
-def test_search__install_directly_from_search(feed_pt, iw_config, run):
+def test_search__install_one(feed_pt, iw_config, run):
     feed_pt(' \r\r')  # space, enter, enter
-    assert (
-        run('search molinari --source curse').output
-        == f'✓ curse:molinari\n  installed 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""}\n'
+    assert run('search molinari --source curse').output == dedent(
+        f'''\
+        ✓ curse:molinari
+          installed 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""}
+        '''
     )
 
 
-def test_search__install_multiple(feed_pt, iw_config, run):
+def test_search__install_multiple_conflicting(feed_pt, iw_config, run):
     feed_pt(' \x1b[B \r\r')  # space, arrow down, space, enter, enter
     assert run('search molinari').output in {
-        f'''\
-✓ curse:molinari
-  installed 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""}
-✗ wowi:13188-molinari
-  package folders conflict with installed package Molinari
-    (curse:20338)
-''',
-        f'''\
-✓ wowi:13188-molinari
-  installed 90000.73-Release
-✗ curse:molinari
-  package folders conflict with installed package Molinari
-    (wowi:13188)
-''',
+        dedent(
+            f'''\
+            ✓ curse:molinari
+              installed 90000.73-Release{"-classic" if iw_config.game_flavour is Flavour.vanilla_classic else ""}
+            ✗ wowi:13188-molinari
+              package folders conflict with installed package Molinari
+                (curse:20338)
+            '''
+        ),
+        dedent(
+            f'''\
+            ✓ wowi:13188-molinari
+              installed 90000.73-Release
+            ✗ curse:molinari
+              package folders conflict with installed package Molinari
+                (wowi:13188)
+            '''
+        ),
     }
 
 
+def test_changelog_output_with_arg(molinari_and_run):
+    assert molinari_and_run('view-changelog curse:molinari').output.startswith(
+        '<h3>Changes in 90000.73-Release:</h3>'
+    )
+
+
+def test_argless_changelog_output(molinari_and_run):
+    assert molinari_and_run('view-changelog').output.startswith(
+        'curse:molinari:\n  <h3>Changes in 90000.73-Release:</h3>'
+    )
+
+
 @pytest.mark.parametrize(
     'command, exit_code',
     [
@@ -498,9 +529,8 @@ def test_can_list_with_substr_match(molinari_and_run):
 
 
 def test_json_export(iw_config, molinari_and_run):
-    export_json = iw_config.config_dir.parent / 'export.json'
     output = molinari_and_run('list -f json').output
-    assert MultiPkgModel.parse_raw(output).__root__[0].name == 'Molinari'
+    assert PkgList.parse_raw(output).__root__[0].name == 'Molinari'
 
 
 def test_show_version(run):
