@@ -39,18 +39,14 @@ def test_make_zip_member_filter_discards_names_with_prefix_not_in_dirs():
     assert list(map(is_member, ['a/', 'b/', 'aa/', 'bb/'])) == [False, True, False, False]
 
 
-def test_loading_toc_from_path(fake_addon):
-    TocReader.from_path(fake_addon / 'FakeAddon.toc')
+def test_loading_toc_from_addon_path(fake_addon):
+    TocReader.from_addon_path(fake_addon)
     with pytest.raises(FileNotFoundError):
-        TocReader.from_path(fake_addon / 'MissingToc.toc')
-
-    TocReader.from_parent_folder(fake_addon)
-    with pytest.raises(FileNotFoundError):
-        TocReader.from_parent_folder(fake_addon.parent / 'MissingAddon')
+        TocReader.from_addon_path(fake_addon.parent / 'MissingAddon')
 
 
 def test_parsing_toc_entries(fake_addon):
-    toc_reader = TocReader.from_parent_folder(fake_addon)
+    toc_reader = TocReader.from_addon_path(fake_addon)
     assert toc_reader.entries == {
         'Normal': 'Normal entry',
         'Compact': 'Compact entry',
@@ -58,7 +54,7 @@ def test_parsing_toc_entries(fake_addon):
 
 
 def test_toc_entry_indexing(fake_addon):
-    toc_reader = TocReader.from_parent_folder(fake_addon)
+    toc_reader = TocReader.from_addon_path(fake_addon)
     assert toc_reader['Normal'] == 'Normal entry'
     assert toc_reader['Compact'] == 'Compact entry'
     assert toc_reader['Indented'] is None
@@ -67,7 +63,7 @@ def test_toc_entry_indexing(fake_addon):
 
 
 def test_toc_entry_multiindexing(fake_addon):
-    toc_reader = TocReader.from_parent_folder(fake_addon)
+    toc_reader = TocReader.from_addon_path(fake_addon)
     assert toc_reader['Normal', 'Compact'] == 'Normal entry'
     assert toc_reader['Compact', 'Normal'] == 'Compact entry'
     assert toc_reader['Indented', 'Normal'] == 'Normal entry'
@@ -80,7 +76,7 @@ def test_bucketise_bucketises_by_putting_things_in_a_bucketing_bucket():
 
 
 def test_tabulate_spits_out_ascii_table(fake_addon):
-    toc_reader = TocReader.from_parent_folder(fake_addon)
+    toc_reader = TocReader.from_addon_path(fake_addon)
     data = [('key', 'value'), *toc_reader.entries.items()]
     assert tabulate(data) == (
         '  key        value    \n'
@@ -123,10 +119,14 @@ def test_file_uri_to_path_win32_leading_slash_is_stripped():
 
 @pytest.mark.iw_no_mock
 @pytest.mark.asyncio
-async def test_is_outdated_works_in_variety_of_scenarios(monkeypatch, aresponses):
+async def test_is_outdated_works_in_variety_of_scenarios(monkeypatch, aresponses, iw_temp_dir):
+    pypi_version = iw_temp_dir.joinpath('.pypi_version')
+    if pypi_version.exists():
+        pypi_version.unlink()
+
     # 'dev' in version number, version not cached
     with monkeypatch.context() as patcher:
-        patcher.setattr('instawow.utils.get_version', lambda: '0.0.0-dev')
+        patcher.setattr('instawow.__version__', '0.0.0-dev')
         assert await is_outdated() == (False, '')
 
     # Update check disabled, version not cached
@@ -136,7 +136,7 @@ async def test_is_outdated_works_in_variety_of_scenarios(monkeypatch, aresponses
 
     # Endpoint not responsive, version not cached
     with monkeypatch.context() as patcher:
-        patcher.setattr('instawow.utils.get_version', lambda: '0.0.0')
+        patcher.setattr('instawow.__version__', '0.0.0')
         aresponses.add(
             'pypi.org',
             '/pypi/instawow/json',
@@ -147,7 +147,7 @@ async def test_is_outdated_works_in_variety_of_scenarios(monkeypatch, aresponses
 
     # Endpoint responsive, version not cached and version different
     with monkeypatch.context() as patcher:
-        patcher.setattr('instawow.utils.get_version', lambda: '0.0.0')
+        patcher.setattr('instawow.__version__', '0.0.0')
         aresponses.add(
             'pypi.org',
             '/pypi/instawow/json',
@@ -158,7 +158,7 @@ async def test_is_outdated_works_in_variety_of_scenarios(monkeypatch, aresponses
 
     # 'dev' in version number, version cached
     with monkeypatch.context() as patcher:
-        patcher.setattr('instawow.utils.get_version', lambda: '0.0.0-dev')
+        patcher.setattr('instawow.__version__', '0.0.0-dev')
         assert await is_outdated() == (False, '')
 
     # Update check disabled, version cached
@@ -168,7 +168,7 @@ async def test_is_outdated_works_in_variety_of_scenarios(monkeypatch, aresponses
 
     # Endpoint not responsive, version cached
     with monkeypatch.context() as patcher:
-        patcher.setattr('instawow.utils.get_version', lambda: '0.0.0')
+        patcher.setattr('instawow.__version__', '0.0.0')
         aresponses.add(
             'pypi.org',
             '/pypi/instawow/json',
@@ -179,7 +179,7 @@ async def test_is_outdated_works_in_variety_of_scenarios(monkeypatch, aresponses
 
     # Endpoint responsive, version cached and version same
     with monkeypatch.context() as patcher:
-        patcher.setattr('instawow.utils.get_version', lambda: '0.0.0')
+        patcher.setattr('instawow.__version__', '0.0.0')
         aresponses.add(
             'pypi.org',
             '/pypi/instawow/json',
@@ -190,7 +190,7 @@ async def test_is_outdated_works_in_variety_of_scenarios(monkeypatch, aresponses
 
     # Endpoint responsive, version cached and version different
     with monkeypatch.context() as patcher:
-        patcher.setattr('instawow.utils.get_version', lambda: '1.0.0')
+        patcher.setattr('instawow.__version__', '1.0.0')
         aresponses.add(
             'pypi.org',
             '/pypi/instawow/json',
